<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class Address extends Model{
    protected $guarded = [];
    protected $table = "addresses";
    protected $appends = ['is_open','max_delivery_miles'];
    const USER_TYPE_SELLER = 'seller';
	
	
	public function user(){
   		return $this->belongsTo('App\User','user_id');
  	}
	
  	public function getIsOpenAttribute()
  	{
  	    if(empty($this->open_time) || empty($this->close_time)){
  	        return false;
  	    }
  	    
  	    $open = self::isCurrentyOpen($this->open_time, $this->close_time);
  	    
  	    if (!$open || $this->user->is_open == "No") {
  	        return false ;
  	    }
  	    return true;
  	    
  	}
  	
  	public function getMaxDeliveryMilesAttribute()
  	{
  	    $webSetting = Settings::first();
  	    
  	    return $webSetting->max_delivery_miles ?? "0";
  	}
  	
  	public static function isCurrentyOpen($openTime,$closeTime)
  	{
  	    $current_time = strtotime("+1 hour", strtotime(date('H:i')));
  	    
  	    $startTime = strtotime($openTime);
  	    $endTime = strtotime($closeTime);
  	    
  	    if ($current_time >= $startTime && $current_time <= $endTime) {
  	        return true;
  	    }
  	    return false;
  	}
  	
  	public static function getSellerBasedOnDistance($latitude,$longitude)
  	{
  	    $maxDistance = Settings::getMaxDistanceInMiles();
  	    
  	    //converting distance to KM
  	    $maxDistance = $maxDistance * 1.609;
  	    
  	    $addresses = self::select(\Illuminate\Support\Facades\DB::raw('user_id,
                            ( 6367 * acos( cos( radians(' . $latitude . ') )
                            * cos( radians( latitude ) ) * cos( radians( longitude ) -
                            radians(' . $longitude . ') ) + sin( radians(' . $latitude . ') )
                            * sin( radians( latitude ) ) ) ) AS distance'))
                            ->whereRaw('( 6367 * acos( cos( radians(' . $latitude . ') )
                            * cos( radians( latitude ) ) * cos( radians( longitude ) -
                            radians(' . $longitude . ') ) + sin( radians(' . $latitude . ') )
                            * sin( radians( latitude ) ) ) )' . '< ?', $maxDistance)
//                             ->orderBy('distance')
                            ->where('user_type',self::USER_TYPE_SELLER)
                            ->get();
                            
        if (count($addresses) == 0) {
            return false;
        }
        
        $sellerIds = [];
        foreach ($addresses as $address) {
            $sellerIds[] = $address->user_id;
        }
        
        return $sellerIds;
  	}
}
