<?php

namespace App\Http\Controllers;
use App\Batch;
use App\Product;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use App\Sku;
use App\Invoice;
use App\Merchant;
use App\User;
use App\Country;
use App\Settings;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;


class BatchesController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
		if(empty($request->stext)){
		    
		    if(User::isSeller()){
		        
		        $user = User::find(Auth::user()->id);
		        
		        $batches = $user->batches()->with('product.product')
		        ->whereHas('product',function($q){
		            $q->whereHas('product');
		        })
		        ->with('users')
		        ->paginate(30);
		        
		    }else{
		        
		        $batches = Batch::with('product.product')
		        ->whereHas('product',function($q){
		            $q->whereHas('product');
		        })
		        ->orderBy('id','desc')
		        ->with('users')
		        ->paginate(30);
		        
		        
		        //echo "<pre>";print_r($batches);exit;
		    }
		    
		} else {
			
		    $searchType = $request->search_type;
		    
			if(User::isSeller()){
			    
			    $user = User::find(Auth::user()->id);
			    
			    if($searchType == 'batch')
			    {
			        $batches = $user->batches()
			        ->where('batch_number','like','%'.$request->stext.'%')
			        ->with('product.product')
			        ->whereHas('product',function($q){
			            $q->whereHas('product');
			        })
			        ->with('users')
			        ->paginate(30);
			    }
			    
			}else
			{
			    if($searchType == 'batch')
			    {
    			    $batches = Batch::where('batch_number','like','%'.$request->stext.'%')
    			    ->with('product.product')->whereHas('product',function($q){
    			        $q->whereHas('product');
    			    })->paginate(30);
    			    
			    }
			    else if($searchType == 'name')
			    {
			        $stext = $request->stext;
			        $batches = Batch::
			        with('product.product')->whereHas('product',function($q) use ($stext){
			            $q->whereHas('product',function($q2) use ($stext){
			                $q2->where('name','like','%'.$stext.'%');
			            });
			        })->paginate(30);
			    }
			    else if($searchType == 'sku')
			    {
			        $stext = $request->stext;
			        $batches = Batch:: with('product.product')
    			        ->whereHas('product',function($q) use ($stext){
    			            $q->whereHas('product');
    			        })
    			        ->whereHas('product',function($q) use ($stext){
    			                $q->where('sku','like','%'.$stext.'%');
    			        })
    			        ->paginate(30);
			    }
			}
		}
		
		if(User::isSeller()){
		    return view('seller/batches/index',['batches'=>$batches]);
		}
        return view('batches/index',['batches'=>$batches]);
    }
	
	public function create(){
		
		$data['products'] = Product::get();
		
		$data['sellers'] = User::where('role','Seller')->get();
		$data['countries'] = Country::get();
		
		$invoices = Invoice::get();
		$idata = [];
		foreach($invoices as $invoice){
			$idata[$invoice->invoice_number]=$invoice->id;
		}	
		$data['invoices'] = json_encode($idata);
		
		$data['merchants'] = Merchant::get();	
		$data['setting'] = Settings::first();	
		
		
		if(User::isSeller()){
		    return view('seller/batches/create',$data);
		}
		return view('batches/create',$data);
	}
	
	
	
	public function getEditForm($id)
	{
	    $data['products'] = Product::get();
	    $data['batch'] = Batch::find($id);
	    $data['sellers'] = User::where('role','Seller')->get();
	    $data['countries'] = Country::get();
	    
	    $invoices = Invoice::get();
	    $idata = [];
	    foreach($invoices as $invoice){
	        $idata[$invoice->invoice_number]=$invoice->id;
	    }
	    $data['invoices'] = json_encode($idata);
	    
	    $data['merchants'] = Merchant::get();
	    $data['setting'] = Settings::first();
	    
	    if(User::isSeller()){
	        return view('seller/batches/getEditForm',$data);
	    }
	    return view('batches/getEditForm',$data);
	    
	}
	
	public function updateBatch(Request $request)
	{
	    $posted = $request->all();
	    
	    $batch = Batch::where('id',$posted['batch_id'])->first();
	    if(empty($batch)){
	        return redirect()->back()->with('error','Product stock has not been updated! Please try again.');
	    }
	    
	    $batch->batch_number = $posted['batch_number'];
	    $batch->country_id = $posted['country_id'];
	    $batch->quantity = $posted['quantity'];
		$batch->balance_quantity = $posted['quantity'];
	    $batch->product_mrp = $posted['product_mrp'];
	    $batch->weight = $posted['weight'];
	    $batch->weight_unit = $posted['weight_unit'];
	    $batch->margin_ratio = $posted['margin_ratio'];
	    $batch->base_rate = $posted['base_rate'];
	    
	    if($batch->save()){
	        return redirect()->back()->with('success','Stock Update successfully.');
	    }
	    return redirect()->back()->with('error','Stock Not Updated.');
	    
	}
	
	public function store(Request $request){
    	$posted = $request->all();
//        echo "<pre>" ;print_r($posted);exit;
		
		DB::beginTransaction();
		try {
// 			$invoice_no = $posted['invoice_no'];
// 			$invoice_date = $posted['invoice_date'];
// 			$merchant_id = $posted['merchant_id'];
			
// 			//getting invoice details
// 			$invoice = Invoice::where(['merchant_id'=>$merchant_id,'invoice_number'=>$invoice_no,'invoice_date'=>$invoice_date])->first();
// 			if(empty($invoice)){
// 				//saving invoice if not exist
// 				$newInvoice = new Invoice;
				
// 				$newInvoice->invoice_number = $invoice_no;
// 				$newInvoice->merchant_id = $merchant_id;
// 				$newInvoice->invoice_date = $invoice_date;
				
// 				$newInvoice->save();
				
// 				$invoice_id = $newInvoice->id;
// 			} else {
// 				$invoice_id = $invoice->id;
// 			}
			
		    if(User::isSeller()){
		        $sellerId = Auth::user()->id;
		    }else{
		        $sellerId = $posted['seller_id'];
		    }
		    
			$batches = [];
			foreach($posted['product'] as $product){
				$count = Batch::where('sku_id',$product['sku_id'])->count();
				
				
				
				$path = $product['batch']->getRealPath();
				$data = array_map('str_getcsv', file($path));
				
				$CSVbatches = array_slice($data, 0);
				
				foreach($CSVbatches as $key => $CSVbatch)
				{
				    if($key == 0){
				        continue;
				    }
				    if($count > 0 ){
				    	Batch::where('sku_id',$product['sku_id'])->where('seller_id',$sellerId)->update([
				    			'product_mrp' => $CSVbatch[2],
								'margin_ratio' => $CSVbatch[3],
								'base_rate' => $CSVbatch[4]
				  			  ]);
				    }
				    $batches[]=[
				        'product_id'=>$product['product_id'],
				        'sku_id'=>$product['sku_id'],
				        'seller_id' => $sellerId,
				        'batch_number'=>$CSVbatch[0],
				        'quantity'=>$CSVbatch[1],
				        'balance_quantity'=>$CSVbatch[1],
				        'product_mrp'=>$CSVbatch[2],
				        'margin_ratio'=>$CSVbatch[3],
				        'base_rate'=>$CSVbatch[4],
				        'sale_order'=>$count+$key,
				        'enabled'=>'Yes',
				        'created_at'=>Carbon::now(),
				        'updated_at'=>Carbon::now()
				    ]; 
				}
			}
			Batch::insert($batches);
			
			DB::commit();
			return redirect()->back()->with('success','Product stock has been updated!');
		} catch (\Exception $e) {
			DB::rollback();
			return redirect()->back()->with('error','Product stock has not been updated! Please try again.');
		}
		
		
	}
	
	//Download sample batch file
	public function downloadBatchSample()
	{
	    $fileName = 'sample-batch.csv';
	    
	    $headers = array(
	        "Content-type"        => "text/csv",
	        "Content-Disposition" => "attachment; filename=$fileName",
	        "Pragma"              => "no-cache",
	        "Cache-Control"       => "must-revalidate, post-check=0, pre-check=0",
	        "Expires"             => "0"
	    );

	    $columns = array('Batch No.', 'Quantity','MRP','Offset','Selling Price');
	    
	    $callback = function() use($columns) {
	        $file = fopen('php://output', 'w');
	        fputcsv($file, $columns);
	        
	        $row['Batch No.']  = "123";
	        $row['Quantity'] = "10";
	        $row['MRP'] = "100";
	        $row['Offset'] = "2";
	        $row['Selling Price'] = "102";
	        
	        fputcsv($file, array($row['Batch No.'], $row['Quantity'], $row['MRP'], $row['Offset'], $row['Selling Price']));
	        
	        fclose($file);
	    };
	    
	    return response()->stream($callback, 200, $headers);
	}
	
	/*
	public function store(Request $request){
    	//dd($request->all());
		$request->validate([
            'batch_number' => 'required|unique:batches',
            'product_id' => 'required|numeric',
            'mfd_date'=>'required|date',
            'expiry_date'=>'required|date',
            'quantity'=>'required|numeric',
            'margin_ratio'=>'required|numeric',
            'product_mrp'=>'required|numeric',
            'base_rate'=>'required|numeric'
        ]);
		
		$count = Batch::where('product_id',$request->product_id)->count();
		
        $batch = new Batch;
		$batch->fill(collect($request->all())->put('sale_order',$count+1)->put('offer_end_date',$request->offer_end_date==""?'':date('Y-m-d',strtotime($request->offer_end_date)))->put('offer_start_date',date('Y-m-d',strtotime($request->offer_start_date)))->put('balance_quantity',$request->quantity)->forget('_token')->toArray());
		
		if($batch->save()){
			return redirect('/batches')->with('success','New Batch created successfully!');
		} else {
			return redirect('/create')->with('error','New Batch has not been created!');
		}
		
	} */
	
	public function show($id)
    {
    	$batch = Batch::with(['product.images','product.product','invoice.merchant'])->find($id);
		
    	if(User::isSeller()){
    	    return view('seller/batches/show',['batch'=>$batch]);
    	}
		//dd($batch->toArray());
		return view('batches/show',['batch'=>$batch]);
		
    }
	
    public function deleteBatch($id)
    {
        if(Batch::destroy($id)){
            return redirect()->back()->with('success','Stock has been Deleted!');
        } else {
            return redirect()->back()->with('error','Stock has not been Deleted!');
        }
    }
	
	public function markBatchEnabled(Request $request,$id,$enabled){
		
		Batch::where('id',$id)->update(['enabled'=>$enabled]);
		
		return redirect()->back()->with('success','Updated');
	}
	
	/*Validation for Batch Number Unique*/
	public function validateBatchNumber(Request $request,$id=null){
		
		if($id==null){
			$batches = Batch::whereBatchNumber($request->batch_number)->first();
		} else {
			$batches = Batch::whereBatchNumber($request->batch_number)->where('id','!=',$id)->first();
		}
		
		$isAvailable = true;
		if(!empty($batches)){
			$isAvailable = false;
		}
		//$isAvailable = false;
		echo json_encode(array(
		    'valid' => $isAvailable,
		));
	}
	/*Batch Number Validate ends*/
	
	
	
	public function getProductSkusAjax($product_id){
		$skus = Product::where('id',$product_id)->first()->skus()->get();
		echo "<option value=''>-- Choose -- </option>";
		foreach($skus as $sku){
			echo "<option value='".$sku->id."'>".$sku->sku." - ".$sku->variation_name."(".$sku->variation_value.")</option>";
		}
	}
	
	public function getInvoiceDetailsAjax($invoice_id){
		$invoice = Invoice::find($invoice_id);
		if(empty($invoice)){
			echo 0;
		} else {
			$idata = [
				'merchant_id'=>$invoice->merchant_id,
				'invoice_date'=>$invoice->invoice_date,
			];
			
			echo json_encode($idata);
		}
	}
	
	
}
