<?php

namespace App\Http\Controllers;
use App\Cart;
use App\Product;
use Illuminate\Http\Request;
use Validator;
use DB;
use App\Sku;
use Illuminate\Support\Facades\Auth;
use App\Tax;
use App\Settings;


class CartController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }
    
	public function myCart()
	{
	    $data['cart_count'] = Cart::getCartCount();
		$data['my_cart'] = Cart::where(['user_id'=>Auth::user()->id])->whereHas('variation')->with(['variation.currentBatch','variation.product.categories','variation.images'])->paginate(100);
		
		$setting = Settings::first();
		$data['service_charges'] = $setting->service_charges;
		$data['minimum_order_amount'] = $setting->minimum_order_amount;
		return response()->json(['status' => 1, 'message' => 'My Cart', 'data' => $data]);
	}
	
	public function addOrUpdateCart(Request $request)
	{	
		$validator = Validator::make($request->all(), [
                    'variation_id'=>'required|numeric',
                    'quantity'=>'required|numeric'
        ]);
		$fields = array('variation_id','quantity');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }

		$posted = $request->all();
		
		//$logginUser
		$loginUserId = Auth::user()->id;
		
		//find seller (store) of already added batch
		$my_cart = Cart::where(['user_id'=>Auth::user()->id])->first();
		
		if(!empty($my_cart)){
		    
		    //find seller (store) of newly added batch
		    $variation = Sku::find($posted['variation_id']);
		    
		    if(empty($variation)){
		        return response()->json(['status' => 0, 'message' => "Variation not found."]);
		    }
		    
		    $newlyAddedSellerId = $variation->currentBatch->seller_id;
		    
		    //find seller (store) of already added batch
		    $alreadyAddedSellerId = ($my_cart->variation && $my_cart->variation->currentBatch) ? $my_cart->variation->currentBatch->seller_id : "";
		    
// 		    echo "<pre>";print_r( $newlyAddedSellerId);exit;
		    if(!empty($alreadyAddedSellerId)){
        		if($newlyAddedSellerId != $alreadyAddedSellerId && empty($posted['overide_cart']))
        		{
        		    $sameStore = true;
        		    return response()->json(['status' => 0, 'is_different_store' => 'yes','message' => "Items already added from a different store."]);
        		}
        		
        		if($posted['overide_cart'] == "yes" && $newlyAddedSellerId != $alreadyAddedSellerId){
        		    $cartItems = Cart::where('user_id',$loginUserId)->get();
        		    if(!empty($cartItems)){
        		        foreach ($cartItems as $item){
        		            $item->delete();
        		        }
        		    }
        		    
        		}
		    }
    		//echo "<pre>";print_r($cartItems);exit;
		}
		
		$cart = new Cart;
		//$cart->fill(['user_id'=>Auth::user()->id,'product_id'=>$posted['product_id'],'quantity'=>$posted['quantity']]);
		$status = Cart::updateOrCreate(
		    ['user_id'=>$loginUserId,'sku_id'=>$posted['variation_id']],
			['quantity'=>$posted['quantity']]
		);
		if($status){
		    $data['cart_count'] = Cart::getCartCount();
		    $data['my_cart'] = Cart::where(['user_id'=>Auth::user()->id,'sku_id'=>$posted['variation_id']])->first();
			return response()->json(['status' => 1, 'message' => "Product Variation Added / Updated in Cart",'data'=>$data]);
		} else {
			return response()->json(['status' => 0, 'message' => "Not Added."]);
		}
		
	}

	public function removeFromMyCart(Request $request)
	{	
		$posted = $request->all();
		$validator = Validator::make($request->all(), [
                    'cart_id'=>'required|numeric'
        ]);
		$fields = array('cart_id');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
		
		$cart = Cart::find($posted['cart_id']);
		
		if(!$cart){
		    $data['cart_count'] = Cart::getCartCount();
		    $data['my_cart'] = Cart::where(['user_id'=>Auth::user()->id])->whereHas('variation')->with(['variation.currentBatch','variation.product.categories','variation.images'])->get();
			return response()->json(['status' => 0, 'message' => 'Invalid Cart ID', 'data' => $data]);
		}
		
		if($cart->delete()){
		    $data['cart_count'] = Cart::getCartCount();
		    $data['my_cart'] = Cart::where(['user_id'=>Auth::user()->id])->whereHas('variation')->with(['variation.currentBatch','variation.product.categories','variation.images'])->get();
		    return response()->json(['status' => 1, 'message' => 'Successfully Removed from Cart', 'data' => $data]);
		} else {
		    $data['cart_count'] = Cart::getCartCount();
		    $data['my_cart'] = Cart::where(['user_id'=>Auth::user()->id])->whereHas('variation')->with(['variation.currentBatch','variation.product.categories','variation.images'])->get();
			return response()->json(['status' => 0, 'message' => 'Not Successful.', 'data' => $data]);
		}
	}
	
	
}
