<?php
namespace App\Http\Controllers;

use App\Culture;
use Illuminate\Http\Request;

class CultureController extends Controller
{

    public function __construct()
    {
        // $this->middleware('auth');
    }

    public function culturesList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['cultures'] = Culture::where($request->ftype, 'like', '%' . $request->stext . '%')->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['cultures'] = Culture::paginate(400);
            $data['ftype'] = "";
            $data['stext'] = "";
        }

        return view('cultures/culturesList', $data);
    }

    public function addCultures()
    {
        return view('cultures/addCultures');
    }

    public function saveCultures(Request $request)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'name' => 'required'
        ], [
            'name.unique' => 'Name with this pin code already exists.'
        ]);
        // print_r($posted);exit;
        $culture = new Culture();
        $culture->name = $posted['name'];
        $culture->enabled = $posted['enabled'];
        
        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;
            
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $culture->image = $imageUrl;
            }
        }

        if ($culture->save()) {
            return redirect()->route('culturesList')->with('success', 'Culture created successfully.');
        } else {
            return back()->with('error', 'Culture has not been created.');
        }
    }

    public function editCulture($id)
    {
        $culture = Culture::where([
            'id' => $id
        ])->first();
        if (! $culture) {
            return redirect()->route('culturesList')->with('warning', 'Invalid Culture to edit');
        }
        return view('cultures/editCultures', [
            'culture' => $culture
        ]);
    }

    public function saveEditCulture(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'name' => 'unique:cultures,id,' . $id
        ], [
            'name.unique' => 'Name already exists.'
        ]);

        $culture = Culture::where([
            'id' => $id
        ])->first();
        if (! $culture) {
            return redirect()->route('culturesList')->with('warning', 'Invalid culture to edit');
        }
        $culture->name = $posted['name'];
        $culture->enabled = $posted['enabled'];
        
        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/categories/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/categories/' . $imageFileName;
            
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $culture->image = $imageUrl;
            }
        }

        if ($culture->save()) {
            return redirect()->route('culturesList')->with('success', 'Culture updated successfully.');
        } else {
            return back()->with('error', 'Culture has not been modified.');
        }
    }

    public function deleteCulture(Request $request)
    {
        if (Culture::find($request->id)->forceDelete()) {

            return redirect()->back()->with('success', 'Culture has been Deleted!');
        } else {
            return redirect()->back()->with('error', 'Culture has not been Deleted!');
        }
    }

    /**
     * API start here
     */
    public function getCulturesList()
    {
        $data['cultures'] = Culture::where('enabled', 'Yes')->paginate(30);

        if (count($data['cultures']) == 0) {
            return response()->json([
                'status' => 0,
                'message' => "Cultures not found."
            ]);
        }

        return response()->json([
            'status' => 1,
            'message' => "Culture Send Successfully",
            'data' => $data
        ]);
    }
}
