<?php

namespace App\Http\Controllers;
use App\DeliveryAddress;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Validator;
use DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use App\Mail\NominationMail;
use App\NotDeliveryAddress;

class DeliveryAddressesController extends Controller
{
    public function __construct()
    {
        //$this->middleware('auth');
    }
    
    public function deliveryAddressesList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['addresses'] = NotDeliveryAddress::where('postcode', 'like', '%' . $request->stext . '%')->paginate(30);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['addresses'] = NotDeliveryAddress::paginate(30);
            $data['ftype'] = "";
            $data['stext'] = "";
        }
        
        return view('delivery_addresses/notDeliveredAddressesList', $data);
    }
    
    public function importAddresses()
    {
        return view('delivery_addresses/import');
    }
    
    public function saveNotDeliveredAddress(Request $request)
    {
        $fp = file($request->csv_file);
        //echo count($fp);die();
        if(count($fp) > 10000){
            return redirect()->route('deliveryAddressesList')->with('error', 'Can not import more than 10,000 postcodes in one file.');
        }
        
        $path = $request->csv_file->getRealPath();
        $data = array_map('str_getcsv', file($path));
        
        $postcodes = array_slice($data, 0);
        
        foreach($postcodes as $key => $postcode)
        {
            if($key == 0){
                continue;
            }
            $newCode = $postcode[1];
            $old = NotDeliveryAddress::where('postcode',$newCode)->first();
            if(empty($old)){
                $notDeliveryAddress = new NotDeliveryAddress();
                $notDeliveryAddress->postcode = $newCode;
                $notDeliveryAddress->save();
            }
            
        }
        return redirect()->route('deliveryAddressesList')->with('success', 'Not Deliverable Addresses import successfully.');
    }
    
    public function exportSampleAddresses()
    {
        $fileName = 'sample-postcode.csv';
        
        $headers = array(
            "Content-type"        => "text/csv",
            "Content-Disposition" => "attachment; filename=$fileName",
            "Pragma"              => "no-cache",
            "Cache-Control"       => "must-revalidate, post-check=0, pre-check=0",
            "Expires"             => "0"
        );
        
        $columns = array('Sr. No.', 'Postcode');
        
        $callback = function() use($columns) {
            $file = fopen('php://output', 'w');
            fputcsv($file, $columns);
            
            $no = 6;
            for ($i=1;$i<4;$i++) 
            {
                $row['Sr. No.']  = $i;
                $row['Postcode'] = "CA8 " . $no . "HR";
                $no++;
                fputcsv($file, array($row['Sr. No.'], $row['Postcode']));
            }
            
            fclose($file);
        };
        return response()->stream($callback, 200, $headers);
    }
    
    public function addDeliveryAddress()
    {
        return view('delivery_addresses/addDeliveryAddress');
    }
    
    public function saveDeliveryAddress(Request $request)
    {
        $posted = collect($request->all())->toArray();
        
        $request->validate([
            'name' => 'required',
            'pin_code' => 'unique:delivery_addresses'
        ], [
            'pin_code.unique' => 'Address with this pin code already exists.'
        ]);
        //print_r($posted);exit;
        $address = new DeliveryAddress();
        $address->name = $posted['name'];
        $address->pin_code = $posted['pin_code'];
        $address->enabled = $posted['enabled'];
        
        if ($address->save()) {
            return redirect()->route('deliveryAddressesList')->with('success', 'Delivery Address created successfully.');
        } else {
            return back()->with('error', 'Delivery Address has not been created.');
        }
    }
	
    
    
    public function editDeliveryAddress($id)
    {
        $address = DeliveryAddress::where([
            'id' => $id
        ])->first();
        if (! $address) {
            return redirect()->route('deliveryAddressesList')->with('warning', 'Invalid delivery address to edit');
        }
        return view('delivery_addresses/editDeliveryAddress', [
            'address' => $address
        ]);
    }
    
    public function saveEditDeliveryAddress(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();
        
        $request->validate([
            'name' => 'required',
            'pin_code' => 'unique:delivery_addresses'
        ], [
            'pin_code.unique' => 'Address with this pin code already exists.'
        ]);
        
        $address = DeliveryAddress::where([
            'id' => $id
        ])->first();
        if (! $address) {
            return redirect()->route('deliveryAddressesList')->with('warning', 'Invalid delivery address to edit');
        }
        $address->name = $posted['name'];
        $address->pin_code = $posted['pin_code'];
        $address->enabled = $posted['enabled'];
        
        if ($address->save()) {
            return redirect()->route('deliveryAddressesList')->with('success', 'Delivery address updated successfully.');
        } else {
            return back()->with('error', 'Delivery address has not been modified.');
        }
    }
    
    public function deleteDeliveryAddress(Request $request)
    {
        if (NotDeliveryAddress::find($request->id)->forceDelete()) {
            
            return redirect()->back()->with('success', 'Delivery Address has been Deleted!');
        } else {
            return redirect()->back()->with('error', 'Delivery Address has not been Deleted!');
        }
    }
    
    public function markInUse(Request $request,$id,$inUse){
        
        DeliveryAddress::where('id',$id)->update(['in_use'=>$inUse]);
        
        return redirect()->back()->with('success','Updated');
    }
    
    
    /**
     * API start here
     */
    
    public function sendNominationMail(Request $request)
    {
        if(empty($request->post_code) || empty($request->email) || empty($request->cultures)){
            return response()->json(['status' => 0, 'message' => "No data posted", 'data' => ""]);
        }
        
        //$postcode = DeliveryAddress::getPostCodeDetail($request->post_code);
        
//         if($postcode->status != "200"){
//             return response()->json(['status' => 0, 'message' => "Invaild postcode", 'data' => ""]);
//         }
        
       // $deliveryAddress = DeliveryAddress::where('Postcode',$postcode->result->postcode)->first();
        $email_content['email'] = $request->email;
        $email_content['postcode'] = $request->post_code;
        $email_content['cultures'] = explode(",",$request->cultures);
        $email_content['subject'] = "Postcode suggest by user where no active store present : ";
        Mail::to('mariam@my-oja.com')->send(new NominationMail($email_content));
        
//         if(empty($deliveryAddress)){
            
//             $deliveryAddress = new DeliveryAddress();
//             $deliveryAddress->Postcode = $postcode->result->postcode;
//             $deliveryAddress->in_use = "No";
//             $deliveryAddress->Latitude = $postcode->result->latitude;
//             $deliveryAddress->Longitude = $postcode->result->longitude;
//             $deliveryAddress->Easting = $postcode->result->eastings;
//             $deliveryAddress->Northing = $postcode->result->northings;
//             $deliveryAddress->Region = $postcode->result->region;
//             $deliveryAddress->District = $postcode->result->admin_district;
//             $deliveryAddress->Ward = $postcode->result->admin_ward;
//             $deliveryAddress->Country = $postcode->result->country;
//             $deliveryAddress->County = $postcode->result->admin_county;
//             $deliveryAddress->save();
//         }
        return response()->json(['status' => 1, 'message' => "Nomination mail successfully send to admin.", 'data' => ""]);
    }
    
    /*
     * get list of all postcode matching with search keyword
     */
    public function getPostcodeList(Request $request)
    {
        if(empty($request->keyword)){
            return response()->json(['status' => 0, 'message' => "No data posted", 'data' => ""]);
        }
        
        $deliveryAddress = DeliveryAddress::whereRaw('REPLACE (Postcode," ","") LIKE "%'.str_replace(' ','%',$request->keyword).'%"')->select('Postcode')->get();
        
        return response()->json(['status' => 1, 'data' => $deliveryAddress]);
    }
  
}
