<?php

namespace App\Http\Controllers;
use App\Offer;
use Illuminate\Http\Request;
use Validator;
use DB;


class OffersController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
    	if(count($request->all())>1){
			$data['offers'] = Offer::where($request->ftype,'like','%'.$request->stext.'%')->paginate(500);
			
			$data['ftype'] = $request->ftype;
			$data['stext'] = $request->stext;
		} else {
			$data['offers'] = $offers = Offer::paginate(30);
			$data['ftype'] = "";
			$data['stext'] = "";
		}
		
    	
        return view('offers/index',$data);
    }
	
	
	public function create()
    {
    	
        return view('offers/create');
    }
	
	public function store(Request $request){
    	
		$request->validate([
            'name' => 'required',
		    'amount' => 'required',
		    'code' => 'required',
//             'image'=>'required|mimes:jpeg,bmp,png,jpg,svg'
        ],[
        	'name.required'=>"Offer Title is required",
            'amount.required'=>"Amount is required",
            'code.required'=>"Coupon code is required",
//         	'image.required'=>"Image is required",
//         	'image.mimes'=>'Only Images are allowed.'
        	
        ]);
		
// 		print_r($request->all());exit;
		
        $offer = new Offer;
		
		$offer->name = $request->name;
		$offer->amount = $request->amount;
		$offer->discount_type = $request->discount_type;
		$offer->min_amount = $request->min_amount ?? '0';
		$offer->discount_upto = $request->discount_upto;
		$offer->start_date = $request->start_date;
		$offer->end_date = $request->end_date;
		$offer->code = $request->code;
		$offer->enabled = $request->enabled;
		
		//if uploaded file
// 		if($request->hasFile('image')){
// 			$image = $request->file('image');
// 	        $imageFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $image->getClientOriginalExtension();
// 	        $imageUrl="https://gadgetzone.s3-ap-south-1.amazonaws.com/images/offers/".$imageFileName;
// 	        $s3 = \Storage::disk('s3');
// 			$filePath = '/images/offers/' . $imageFileName;
// 	        $imageResult=$s3->put($filePath, file_get_contents($image), 'public');
// 			if($imageResult==1){
// 				$offer->image = $imageUrl;
// 			} 
// 		}
		
		
		
		if($offer->save()){
			return redirect('/offers')->with('success','New Offer created successfully!');
		} else {
			return redirect('/offers')->with('error','New Offer has not been created!');
		}
    }
	
	public function edit($id)
    {
    	$offer = Offer::find($id);
		
        return view('offers/edit',['offer'=>$offer]);
    }
	
	public function update(Request $request,$id){
    	
	    $request->validate([
	        'name' => 'required',
	        'amount' => 'required',
	        'code' => 'required',
	        //             'image'=>'required|mimes:jpeg,bmp,png,jpg,svg'
	    ],[
	        'name.required'=>"Offer Title is required",
	        'amount.required'=>"Amount is required",
	        'code.required'=>"Coupon code is required",
	        //         	'image.required'=>"Image is required",
	    //         	'image.mimes'=>'Only Images are allowed.'
	        
	    ]);
		
        $offer = Offer::find($id);
		
        $offer->name = $request->name;
        $offer->amount = $request->amount;
        $offer->discount_type = $request->discount_type;
        $offer->min_amount = $request->min_amount ?? '0';
        $offer->discount_upto = $request->discount_upto;
        $offer->start_date = $request->start_date;
        $offer->end_date = $request->end_date;
        $offer->code = $request->code;
        $offer->enabled = $request->enabled;
		
		if($offer->save()){
			return redirect('/offers')->with('success','Offer updated successfully!');
		} else {
			return redirect('/offers')->with('error','Offer has not been updated!');
		}
    }

	public function destroy(Request $request,$id){
		//this will soft delete the product
		
		if(Offer::destroy($id)){
			return redirect('/offers')->with('success','Offer has been Deleted!');
		} else {
			return redirect('/offers')->with('error','Offer has not been Deleted!');
		}
	}
	
	//ajax to get new SKU No
	public function getNewCouponCode(Request $request){
	    echo $this->genCouponCode();
	}
	
	//generate new sku no
	public function genCouponCode(){
	    $cc = "OJA".rand(1111,9999);
	    $offer = Offer::where('code',$cc)->first();
	    if(!empty($offer)){
	        genCouponCode();
	    }
        return $cc;
	        
	}
	
	/*
	 * Api Start here
	 */
	
	public function getOffersList()
	{
	    $offers = Offer::where('enabled','Yes')->where('end_date', '>=', date('Y-m-d'))->paginate(20);
	    
	    return response()->json(['status' => 1, 'message' => 'Offers sent successfully', 'data' => $offers]);
	}
	
	
}
