<?php

namespace App\Http\Controllers;
use App\Product;
use App\Batch;
use App\AppDashboard;
use App\Category;
use App\Merchant;
use App\Specification;
use App\ProductImage;
use App\ProductSpecification;
use App\Tax;
use App\Offer;
use App\Sku;
use Illuminate\Http\Request;
use Validator;
use DB;
use App\Wishlist;
use App\Cart;
use View;

use App\Exports\ProductSampleExport;
use App\Imports\ProductImport;
use Maatwebsite\Excel\Facades\Excel;
use App\Country;
use Illuminate\Support\Facades\Auth;
use App\Brand;


class ProductsController extends Controller
{
	
	public $catData=null;
	public $catArray = null;
	
    public function __construct()
    {
        //$this->middleware('auth')->except('appDashboard');
		//$this->middleware('OptionalAuth')->only(['appDashboard']);
		
		$catTree = Category::whereNull('parent_id')->with(['categories'=>function($query){
    		$query->where('enabled','Yes');
    	}])->orderBy('sort_order','asc')->get()->toArray();
		
		$this->catData = $this->genCatList($catTree);
		$this->catArray = $catTree;
		
		
    }


	public function importProducts(Request $request){
		
		//$resp = Excel::import(new ProductImport, $request->file('iProducts'));
		//$import = new ProductImport();
		//$import->import($request->file('iProducts'));
		
		//dd($import->errors());
		
		
		try {
		    //$resp = Excel::import(new ProductImport, $request->file('iProducts'));
			$import = new ProductImport();
			$import->import($request->file('iProducts'));
			return back()->with('success',"Data Imported Successfully.");
		} catch (\Maatwebsite\Excel\Validators\ValidationException $e) {
		     $failures = $e->failures();
		     $errors = [];
		     foreach ($failures as $k=>$failure) {
		     	$errors[$k]['Rows'] = $failure->row();
		        //$errors[$k]['attributes'] = $failure->attribute(); // either heading key (if using heading row concern) or column index
		        $errors[$k]['errors'] = $failure->errors(); // Actual error messages from Laravel validator
		        //$errors[$k]['values'] = $failure->values(); // The values of the row that has failed.
		     }
			 //$errors = json_encode($errors);
			 //return back()->with('error',"<b>".$errors."</b>");
			 
			$mErrors=[];
			foreach($errors as $k=>$error){
				foreach($error['errors'] as $err){
					$mErrors[$error['Rows']][] = $err;
				}
			}
			foreach($mErrors as $row=>$errors){
				$mErrors[$row]= implode(', ',$errors);
			}
			
			$errorHtml = View::make('products/importStatus',['errors'=>$mErrors]);
			$errorHtml = $errorHtml->render();
			return back()->with('importError',$errorHtml);
		}
		
		return view('products/importStatus',['errors'=>$mErrors]);
	}
	

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
    	$posted = [];
    	//$products = Product::with('productSpecifications')->with('categories')->with('images')/*->with('batches')->with('currentBatch')*/;
		
		$skus = Sku::whereHas('product')->with(['product.categories','product.tax','images','productSpecifications','currentBatch','batches'])->orderBy('id','desc');
		
    	if(!empty($request->all())){
    		$posted = $request->all();
			//dd($posted);
			$pname = @$posted['product_name'];
			$sku = @$posted['product_sku'];
			$batch_number = @$posted['batch_number'];
			$from_price = @$posted['price_from'];
			$to_price = @$posted['price_to'];
			$category = @$posted['category'];
			
			
			if(!empty($pname)){
				$skus->whereHas('product',function($q)use($pname){
					$q->where('name','like','%'.$pname.'%');
				});
			}
			
			if(!empty($sku)){
				$skus->where('sku','like','%'.$sku.'%');
			}
			
			if(!empty($batch_number)){
				$skus->whereHas('batches',function($q)use($batch_number){
					$q->where('batch_number','like','%'.$batch_number.'%');
				});
			}
			
			if(!empty($from_price) && !empty($to_price)){
				$skus->whereHas('batches',function($q)use($from_price,$to_price){
					$q->whereBetween('base_rate',[$from_price,$to_price]);
				});
			}
			
			if(!empty($category)){
				$skus->whereHas('product',function($q)use($category){
					$q->whereHas('categories',function($qq)use($category){
						$qq->where('category_id',$category);
						$qq->orWhere('parent_id',$category);
					});
				});
			}
			
    	}
		
		$skus = $skus->paginate(30);
		//dd($skus->toArray());
		
		
		return view('products/index',['products'=>$skus,'categories'=>$this->catArray,'posted'=>$posted,'menubtn'=>'productsli']);
    }
	
	
	public function create()
    {
    	$data = [
    		'catTree'=>$this->catData,
    		'catArray'=>$this->catArray,
    		'merchants'=>Merchant::where('enabled','Yes')->get(),
    		'taxes'=>Tax::where('enabled','Yes')->get(),
    		'products'=>Product::where('enabled','Yes')->get(),
    		'specifications'=>Specification::whereNull('parent_id')->where('enabled','Yes')->get(),
    		'menubtn'=>'productsli',
    	    'countries' => Country::get(),
    	    'brands' => Brand::where('enabled','Yes')->get()
    	];
		//dd($this->catData);
        return view('products/create',$data);
    }
	
	
	public function store(Request $request)
	{
		$posted = collect($request->all())->toArray();
		//dd($posted);
		
// 		echo "<pre>";print_r($request->all());exit;
		
		DB::beginTransaction();
				
		$catIds = $posted['category_id'];
		unset($posted['category_id']);
		
		$product = $posted;
		
		if($product['taxable']=='No'){
			$product['tax_id']=null;
		}
		
		if(!empty($posted['related_products'])){
			$product['related_products'] = implode(',',$posted['related_products']);
		}
		
		$skus = $product['sku'];
		unset($product['sku']);
		//dd($skus);
		
		try{
			//uploading related file
			if($request->hasFile('related_files')){
				$rFile = $request->file('related_files');
		        $rFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $rFile->getClientOriginalExtension();
		        $rFileUrl="https://ojaprod.s3.us-east-2.amazonaws.com/products/".$rFileName;
		        $s3 = \Storage::disk('s3');
				$filePath = '/products/' . $rFileName;
		        $imageResult=$s3->put($filePath, file_get_contents($rFile), 'public');
				if($imageResult==1){
					$product['related_files'] = $rFileUrl;
				} 
			} 
			
			$newProduct = new Product();
				
			$newProduct->fill($product);
			$newProduct->save();
			$newProduct->categories()->attach($catIds); //attaching the categories to product
			
			foreach($skus as $skuData){

				$specs = $skuData['specification'];
				//dd($specs[0]['heading']);
				$photos = $skuData['photos'];
				
				$pSku= new Sku;
				$pSku->product_id = $newProduct->id;
				$pSku->sku = $skuData['sku_number'];
				$pSku->variation_name = $skuData['variation_name'];
				$pSku->variation_value = $skuData['variation_value'];
				
				$pSku->save();
				
				$pSpecs=[];
				if(!empty($specs)){
					$ppspec=[];
					foreach($specs as $k=>$spec){
						foreach($specs[$k]['heading'] as $sp){
							if(!$specs[$k]['heading'][0] == null){
								$ppspec[$sp][]=$spec;
							}
						}
					}
					
					foreach($ppspec as $headId=>$nspec){
						$specHeadData = Specification::find($headId);
						
						$pspec = new ProductSpecification;
						$pspec->product_id = $newProduct->id;
						$pspec->sku_id = $pSku->id;
						$pspec->name = $specHeadData->name;
						
						$pspecA=[];
						if($pspec->save()){
							foreach($nspec as $spec){
							    
							    $pspecA=[];
								foreach($spec['name'] as $k=>$name){
									$pspecA[]=[
										'product_id'=>$newProduct->id,
										'parent_id'=>$pspec->id,
										'name'=>$name,
										'value'=>$spec['value'][$k],
										'use_in_filters'=>$spec['use_in_filters'][$k]
									];
								}
								ProductSpecification::insert($pspecA);	
							}
											
						}
					}
					
					
				}
				
				$pPhotos=[];
				foreach($photos as $image){
					
			        $imageFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $image->getClientOriginalExtension();
			        $imageUrl="https://ojaprod.s3.us-east-2.amazonaws.com/products/".$imageFileName;
			        $s3 = \Storage::disk('s3');
					$filePath = '/products/' . $imageFileName;
					
//             Compress image using tinify
// 				\Tinify\setKey("270QBN29NtRsKxFWHzb5YdYtsfXLbnCy");
// 				$sourceData = file_get_contents($image);
// 				$resultData = \Tinify\fromBuffer($sourceData)->toBuffer();
					
			        $imageResult=$s3->put($filePath, file_get_contents($image), 'public');
					if($imageResult==1){
						$pPhotos[]=[
							'product_id'=>$newProduct->id,
							'sku_id'=>$pSku->id,
							'name'=>$imageUrl
						];
					}
				}
				$pImages = ProductImage::insert($pPhotos);
			
			}
			DB::commit();
			
			return redirect('/products')->with('success','Product has been created!');
		} catch (\Exception $e) {
			DB::rollback();
			dd($e);
			return redirect('/products/create')->with('error','Product has not been created! Please try again.');
		}
		
	
	}
	
		
	public function duplicate($id)
    {
    	$data = [
    		'catTree'=>$this->catData,
    		'catArray'=>$this->catArray,
    		'merchants'=>Merchant::where('enabled','Yes')->get(),
    		'taxes'=>Tax::where('enabled','Yes')->get(),
    		'products'=>Product::where('enabled','Yes')->get(),
    		'specifications'=>Specification::whereNull('parent_id')->where('enabled','Yes')->get(),
    		'product'=>Product::with('productSpecifications')->with('categories')->with('currentBatch')->with('images')->where('id',$id)->first()
    	];
		
		$catids = [];
		foreach($data['product']->categories as $cat){
			$catids[]=$cat->id;
		}
		
		$data['pCats'] = $catids;
		
        return view('products/duplicate',$data);
    }
	
	
	public function show($id)
    {
        return view('products/show');
    }
	
	public function edit($id)
    {
    	
		
    	$data = [
    		'catTree'=>$this->catData,
    		'catArray'=>$this->catArray,
    		'merchants'=>Merchant::where('enabled','Yes')->get(),
    		'taxes'=>Tax::where('enabled','Yes')->get(),
    		'products'=>Product::where('enabled','Yes')->get(),
    		'specifications'=>Specification::whereNull('parent_id')->with('specifications')->where('enabled','Yes')->get(),
    		'product'=>Product::with(['skus.productSpecifications','skus.images','categories'])->where('id',$id)->first(),
    	    'brands' => Brand::where('enabled','Yes')->get()
    	];
		
		
		$catids = [];
		foreach($data['product']->categories as $cat){
			$catids[]=$cat->id;
		}
		
		$data['pCats'] = $catids;
		$data['menubtn'] = 'productsli';
		
		//dd($data);
		
        return view('products/edit',$data);
    }

	public function update(Request $request,$id){
		$posted = collect($request->all())->toArray();
		//dd($posted);
		
		
		DB::beginTransaction();
				
		$catIds = $posted['category_id'];
		unset($posted['category_id']);
		
		$product = $posted;
		
		if($product['taxable']=='No'){
			$product['tax_id']=null;
		}
		
		
		if(!empty($posted['related_products'])){
			$product['related_products'] = implode(',',$posted['related_products']);
		}
		
		$skus = $product['sku'];
		unset($product['sku']);
		//dd($skus);
		$pp=[];
		try{
			//uploading related file
			if($request->hasFile('related_files')){
				$rFile = $request->file('related_files');
		        $rFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $rFile->getClientOriginalExtension();
		        $rFileUrl="https://ojaprod.s3.us-east-2.amazonaws.com/products/".$rFileName;
		        $s3 = \Storage::disk('s3');
				$filePath = '/products/' . $rFileName;
		        $imageResult=$s3->put($filePath, file_get_contents($rFile), 'public');
				if($imageResult==1){
					$product['related_files'] = $rFileUrl;
				} 
			} 
			
			$newProduct = Product::find($id);
				
			$newProduct->fill($product);
			$newProduct->save();
			$newProduct->categories()->detach();
			$newProduct->categories()->attach($catIds); //attaching the categories to product
			
			foreach($skus as $kkk=>$skuData){
			    
			    //echo"<pre>";print_r($skuData);exit;
				
				if($skuData['id']!='new'){
					//deleting existing specifications to create new specs
					$skuSpecs = ProductSpecification::where('sku_id',$skuData['id'])->get();
					foreach($skuSpecs as $skSpec){
						ProductSpecification::destroy($skSpec->id);
					}
					//deleting ends here
				}
				//dd($skuData);
				$specs = $skuData['specification'];
				$photos = $skuData['photos'];
				
				if($skuData['id']=='new'){
					$pSku= new Sku;
					$pSku->sku = $skuData['sku_number'];
				} else {
					$pSku= Sku::find($skuData['id']);
				}
				
				
				$pSku->product_id = $newProduct->id;
				$pSku->variation_name = $skuData['variation_name'];
				$pSku->variation_value = $skuData['variation_value'];
				
				$pSku->save();
				
				$pSpecs=[];
				if(!empty($specs)){
					$ppspec=[];
					foreach($specs as $k=>$spec){
						foreach($specs[$k]['heading'] as $sp){
							if(!$specs[$k]['heading'][0] == null){
								$ppspec[$sp][]=$spec;
							}
						}
					}
					$pp[]=$ppspec;
					
					foreach($ppspec as $headId=>$nspec){
						
						$specHeadData = Specification::find($headId);
						
						$pspec = new ProductSpecification;
						$pspec->product_id = $newProduct->id;
						$pspec->sku_id = $pSku->id;
						$pspec->name = $specHeadData->name;
						
						$pspecA=[];
						if($pspec->save()){
							foreach($nspec as $spec){
								foreach($spec['name'] as $k=>$name){
									$pspecA[]=[
										'product_id'=>$newProduct->id,
										'parent_id'=>$pspec->id,
										'name'=>$name,
										'value'=>$spec['value'][$k],
										'use_in_filters'=>$spec['use_in_filters'][$k]
									];
								}
								ProductSpecification::insert($pspecA);	
							}
											
						}
					} 
					
					
				}
				
				if(!empty($photos['photos'])){
				$pPhotos=[];
					foreach($photos['photos'] as $phkey=>$image){
						
				        $imageFileName = str_replace(" ","",str_replace(".","",microtime())) . '.' . $image->getClientOriginalExtension();
				        $imageUrl="https://ojaprod.s3.us-east-2.amazonaws.com/products/".$imageFileName;
				        $s3 = \Storage::disk('s3');
						$filePath = '/products/' . $imageFileName;
				        $imageResult=$s3->put($filePath, file_get_contents($image), 'public');
						if($imageResult==1){
							if($photos['id'][$phkey]=='new'){
								$pPhotos[]=[
									'product_id'=>$newProduct->id,
									'sku_id'=>$pSku->id,
									'name'=>$imageUrl
								];
							} else {
								$pPhoto=ProductImage::find($photos['id'][$phkey]);
								$pPhoto->name = $imageUrl;
								$pPhoto->save();
							}
							
						}
					}
					$pImages = ProductImage::insert($pPhotos);
				}
			}
			
		
			
			
			DB::commit();
			
			return redirect('/products')->with('success','Product has been updated!');
		} catch (\Exception $e) {
			DB::rollback();
			
			//dd($e);
			return redirect()->route('products.edit',$id)->with('error','Product has not been updated! Please try again.');
		}
		
	
	}
	
	public function skudetails($sku_id){
		
		$sku = Sku::with(['images','productSpecifications','product.categories','product.tax','batches.invoice.merchant','currentBatch'])->find($sku_id);
		if(empty($sku)){
			return redirect()->route('products.index')->with('warning','Invalid SKU details. Kindly check and try again.');
		}
		$data=[
			'sku'=>$sku
		];
		//dd($sku->toArray());
		return view('products/skudetails',$data);
	}
	
	public function updateSkuBatchOrder(Request $request){
		
		$batches = $request->batches;
		
		try{
			foreach($batches as $position=>$batchId){
				Batch::where(['id'=>$batchId])->update(['sale_order'=>$position]);
			}
			echo 1;
		} catch (Exception $e){
			echo 0;
		}
		
	}
	
	
	
	public function deleteProductSku($id){
		if(Sku::destroy($id)){
			return redirect()->back()->with('success','Product SKU has been deleted!');
		} else {
			return redirect()->back()->with('error','Product SKU has not been deleted! Please try again');
		}
	}
	
	public function deleteProductSkuAjax($id){
		if(Sku::destroy($id)){
			echo 1;
		} else {
			echo 0;
		}
	}
	
	public function deleteSpecificationAjax($id){
		if(ProductSpecification::destroy($id)){
			echo 1;
		} else {
			echo 0;
		}
	}
	public function deleteImageAjax($id){
		if(ProductImage::destroy($id)){
			echo 1;
		} else {
			echo 0;
		}
	}

	
	
	
	public function markPopular(Request $request,$id,$popular){
		//dd($request->all());
		
		Product::where('id',$id)->update(['popular'=>$popular]);
		
		return redirect()->back()->with('success','Updated');
	}
	
	public function destroyProduct(Request $request,$id){
		//this will soft delete the product
		
		if(Product::destroy($id)){
			return redirect()->back()->with('success','Product has been Deleted!');
		} else {
			return redirect()->back()->with('error','Product has not been Deleted!');
		}
		
		
	}
	
	
	
	public function genCatList($tree,$saperator="",$cTree=[]){
		
		foreach($tree as $k=>$row){
			
			$cTree[]=array('id'=>$row['id'],'name'=>$saperator.$row['name']);
			$cTree=$this->genCatList($row['categories'],$saperator.$row['name']." &mdash;> ",$cTree);
        }
		
		
		return $cTree;
	}
	
	
	public function exportSample(){
	    return Excel::download(new ProductSampleExport, 'ProductsUploadTemplate'.date('Ymdhis').'.xlsx');
	}

	public function deleteProductImage(Request $request,$id){
		$pImage = ProductImage::find($id);
		if($pImage->delete()){
			return 1;
		} else {
			return 0;
		}
	}
	
	public function deleteProductSpecification(Request $request,$id){
		
		$pSpec = ProductSpecification::find($id);
		try{
			if($pSpec->delete()){
				if(!empty($request->parent)){
					ProductSpecification::where('parent_id',$id)->delete();
				}
				return 1;
			} else {
				return 0;
			}
		} catch (\Exception $e) {
			return 0;
		}
		
		
	}
	
	//ajax method to validate SKU
	public function validateProductId(Request $request,$id=null){
		
		if($id==null){
			$products = Product::where('product_code',$request->code)->whereDeletedAt(NULL)->first();
		} else {
			$products = Product::where('product_code',$request->code)->whereDeletedAt(NULL)->where('id','!=',$id)->first();
		}
		
		$isAvailable = true;
		if(!empty($products)){
			$isAvailable = false;
		}
		
		echo json_encode(array(
		    'valid' => $isAvailable,
		));
	}
	
	//ajax method to validate SKU
	public function validateProductSku(Request $request,$id=null){
		
		if($id==null){
			$products = Sku::whereSku($request->sku)->whereDeletedAt(NULL)->first();
		} else {
			$products = Sku::whereSku($request->sku)->whereDeletedAt(NULL)->where('id','!=',$id)->first();
		}
		
		$isAvailable = true;
		if(!empty($products)){
			$isAvailable = false;
		}
		
		echo json_encode(array(
		    'valid' => $isAvailable,
		));
	}
	
	
	
	


	//set order of product batches
	public function updateProductBatchOrder(Request $request){
		$posted = $request->data;
		$product_id = $request->product_id;
		try{
			foreach($posted as $position=>$batchId){
				Batch::where(['id'=>$batchId,'product_id'=>$product_id])->update(['sale_order'=>$position]);
			}
			
			echo 1;
		} catch (Exception $e){
			echo 0;
		}
	}
	
	public function getSpecs(Request $request){
		
		$specs = Specification::where('parent_id',$request->parent_id)->get();
		
		foreach($specs as $spec){
			echo "<option>".$spec->name."</option>";
		}
		if(count($specs)==0){
			echo "<option value=''>No Specification Found</option>";
		}
	}
	
	
	//generate new sku no
	public function genSkuNo(){
		$sku = "GM".rand(11111111,99999999)."S";
		$sk = Sku::where('sku',$sku)->first();
		if(!empty($sk)){
			genSkuNo();
		}
		return $sku;
		
	}
	
	//ajax to get new SKU No
	public function getNewSku(Request $request){
		echo $this->genSkuNo();
	}
	
	
	/*API Method*/
	
	public function appDashboard(Request $request){
		
		
		
		$categories = Category::where('name','!=','Uncategorized')->where(function($query){
			$query->where('parent_id','0')
				  ->orWhere('parent_id','')
				  ->orWhereNull('parent_id');
		})->where('enabled','Yes')->orderBy('sort_order','asc')->get();
		
		$data['cart_count'] = Cart::getCartCount();
		
		$data['categories'] = $categories;
		
		$data['sections'] = [
		    'popular'=>Product::where('popular','Yes')->with(['brand','categories','variations.currentBatch','variations.images','variations.productSpecifications','tax','variations.cart','variations.wishList'])
		    ->whereHas('categories',function($query){
				$query->where('name','!=','Uncategorized');
			})->where('enabled','Yes')->take(4)->get(),
			
			'offers'=>Product::where('popular','Yes')->with(['brand','categories','variations.currentBatch','variations.images','variations.productSpecifications','tax','variations.cart','variations.wishList'])
			->whereHas('categories',function($query){
				$query->where('name','!=','Uncategorized');
			})->where('enabled','Yes')->take(6)->get(),
			
			'deals'=>Offer::take(4)->get()
		];
		
		
		
	 	return response()->json(['status' => 1, 'message' => 'Application Dashboard', 'data' => $data]);
	}
	
	public function productsOnSale(){
	    $data['products'] = Product::where('popular','Yes')->with(['brand','tax','categories','variations.currentBatch','variations.images'/*,'variations.productSpecifications'*/,'variations.cart','variations.wishList'])
            	->whereHas('variations',function($query){
            	    $query->whereHas('currentBatch');
            	})
		      ->whereHas('categories',function($query){
				$query->where('name','!=','Uncategorized');
			})->where('enabled','Yes')->paginate(12);
		$data['cart_count'] = Cart::getCartCount();
	 	return response()->json(['status' => 1, 'message' => 'Products on Sale', 'data' => $data]);
	}
	
	public function popularProducts()
	{
	    $data['products'] = Product::where('popular','Yes')
	           ->with(['brand'])
    		->whereHas('variations',function($query){
    		    $query->whereHas('currentBatch');
    		})
		  ->whereHas('categories',function($query){
				$query->where('name','!=','Uncategorized');
			})->where('enabled','Yes')->paginate(15);
			
		$data['cart_count'] = Cart::getCartCount();
	 	return response()->json(['status' => 1, 'message' => 'All Popular Products', 'data' => $data]);
	}

	public function productsOfSubCategory(Request $request)
	{
	    if(empty($request->is_all)){
	        return response()->json(['status' => 0, 'message' => 'Is all param can\'t empty']);
	    }
	    
	    if(empty($request->seller_id)){
	        return response()->json(['status' => 0, 'message' => 'Seller id is required.']);
	    }
	    $sellerId = $request->seller_id;
	    
	    if(empty($request->sub_cat_id) && ($request->is_all == 'no')){
	        return response()->json(['status' => 0, 'message' => 'Sub Category id can\'t be empty']);
	    }
	    
	    if(empty($request->parent_cat_id) && ($request->is_all == 'yes')){
	        return response()->json(['status' => 0, 'message' => 'Parent Category id can\'t be empty']);
	    }
	    
	    if($request->is_all == 'yes'){
	        $parentId = $request->parent_cat_id;
	        $subCatIds = Category::where('parent_id',$parentId)->select('id')->get()->toArray();
	        
// 	        $data['products'] = Product::with(['variations.currentBatch','variations.images','variations.cart','variations.wishList'])
//         	        ->whereHas('variations',function($query){
//         	            $query->whereHas('currentBatch');
//         	        })
//         	        ->whereHas('categories',function($query)use($subCatIds){
//         	            $query->where('name','!=','Uncategorized');
//         	            $query->whereIn('category_id',$subCatIds);
//         	        })->where('enabled','Yes')->paginate(10);
        	        
	        
	        $data['products'] = Product::with(['brand','variations.currentBatch','variations.images','variations.cart','variations.wishList'])
	        ->whereHas('variations',function($query) use ($sellerId){
	            $query->whereHas('currentBatch', function ($query2) use ($sellerId) {
	                $query2->where('seller_id', $sellerId);
	            });
	        })
	        ->with(['variations'=>function($query) use ($sellerId){
	            $query->with(['currentBatch'=> function ($query2) use ($sellerId) {
	                $query2->where('seller_id', $sellerId);
	            }]);
	        }])
	        ->whereHas('categories',function($query)use($subCatIds){
	            $query->where('name','!=','Uncategorized');
	            $query->whereIn('category_id',$subCatIds);
	        })->where('enabled','Yes')->paginate(10);
	        
	        
	    }else{
	        $cat_id = $request->sub_cat_id;
	        
// 	        $data['products'] = Product::with(['variations.currentBatch','variations.images','variations.cart','variations.wishList'])
//     	        ->whereHas('variations',function($query){
//     	            $query->whereHas('currentBatch');
//     	        })
//     	        ->whereHas('categories',function($query)use($cat_id){
//     	            $query->where('name','!=','Uncategorized');
//     	            $query->where('category_id',$cat_id);
//     	        })->where('enabled','Yes')->paginate(10);
    	        
    	        
	        $data['products'] = Product::with(['brand','variations.currentBatch','variations.images','variations.cart','variations.wishList'])
    	        ->whereHas('variations',function($query) use ($sellerId){
    	            $query->whereHas('currentBatch', function ($query2) use ($sellerId) {
    	                $query2->where('seller_id', $sellerId);
    	            });
    	        })
    	        ->with(['variations'=>function($query) use ($sellerId){
    	            $query->with(['currentBatch'=> function ($query2) use ($sellerId) {
    	                $query2->where('seller_id', $sellerId);
    	            }]);
    	        }])
    	        ->whereHas('categories',function($query)use($cat_id){
    	            $query->where('name','!=','Uncategorized');
    	            $query->where('category_id',$cat_id);
    	        })->where('enabled','Yes')->paginate(10);
    	        
    	        
    	        //filter options for this screen
//     	        $data['filterOptions'] = $this->getFilterOptions($cat_id);
	    }
		
    	$data['cart_count'] = Cart::getCartCount();
	 	return response()->json(['status' => 1, 'message' => 'Products by Category', 'data' => $data]);
	}
	
	public function getAllProductsList(Request $request)
	{
	    $data['cart_count'] = Cart::getCartCount(); 
	    
	    if(isset($request->seller_id)){
	        
	        $sellerId = $request->seller_id;
	        
	        $data['products'] = Product::
	        with(['brand',
	            'categories',
	            'variations.images',
	            'variations.cart','variations.wishList'
	        ])->where('enabled','Yes')
	        ->whereHas('variations', function ($query2) use ($sellerId) {
	            $query2->whereHas('currentBatch', function ($query3) use ($sellerId) {
	                $query3->where('seller_id', $sellerId);
	                $query3->whereHas('users',function($query4){
	                    $query4->where('store_exist','Yes');
	                });
	            });
	        })
	            ->orderBy('id','DESC')->paginate(15);
	    }else{
	        
	        $data['products'] = Product::whereHas('variations',function($query){
                    	            $query->whereHas('currentBatch',function($query2){
                    	                $query2->whereHas('users',function($query3){
                    	                    $query3->where('store_exist','Yes');
                    	                });
                    	            });
	                            })->with(['brand',
                    	            'variations.currentBatch','categories',
                    	            'variations.images',
                    	            'variations.cart','variations.wishList'
                    	        ])
                    	        ->where('enabled','Yes')->paginate(15);
	    }
	    
	    
	    if(count($data['products']) == 0){
	        return response()->json(['status' => 0, 'message' => "Products not found.", 'data' => $data]);
	    }
	    
	    return response()->json(['status' => 1, 'message' => 'List of all products.', 'data' => $data]);
	}
	
	public function productDetails($id){
	    $data['product'] = Product::with(['categories','country','variations.currentBatch','variations.currentBatch.country',
                                        'variations.images','brand',
	                                   'variations.cart','variations.wishList'
                    	    ])->whereHas('variations',function($query){
                    	        $query->whereHas('currentBatch');
                    	    })->where('id',$id)
                    	    ->first();
                    	    	    
	    $data['cart_count'] = Cart::getCartCount();
		return response()->json(['status' => 1, 'message' => 'Product Details', 'data' => $data]);
	}
	
	public function getRelatedProducts(Request $request)
	{
	    if(empty($request->cat_id) || empty($request->seller_id)){
	        return response()->json([
	            'status' => 0,
	            'message' => "Store id and category id is required."
	        ]);
	    }
	    if(empty($request->product_id)){
	        return response()->json([
	            'status' => 0,
	            'message' => "Current viewed product is required."
	        ]);
	    }
	    $cat_id = $request->cat_id;
	    $seller_id = $request->seller_id;
	    
	    $data['related_products'] = Product::with(['brand','variations.currentBatch','variations.images','variations.cart','variations.wishList'])
	    ->whereHas('categories',function($query)use($cat_id){
	        $query->where('name','!=','Uncategorized');
	        $query->where('category_id',$cat_id);
	    })->whereHas('variations',function($query) use ($seller_id){
	        $query->whereHas('currentBatch', function ($query2) use ($seller_id) {
	            $query2->where('seller_id', $seller_id);
	        });
	    })->where('enabled','Yes')
	    ->where('id','!=',$request->product_id)->orderBy('id','DESC')->get()->take(10);
	    
	    return response()->json(['status' => 1, 'message' => 'Related Product', 'data' => $data]);
	}
	
	public function addToMyWishlist(Request $request)
	{
		$posted = $request->all();
		$validator = Validator::make($request->all(), [
                    'variation_id'=>'required|numeric'
        ]);
		$fields = array('variation_id');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
		
		$wishList = new Wishlist;
		$wishList->fill(['user_id'=>Auth::user()->id,'sku_id'=>$posted['variation_id']]);
		$status = Wishlist::updateOrCreate(
			['user_id'=>Auth::user()->id,'sku_id'=>$posted['variation_id']],
			['user_id'=>Auth::user()->id,'sku_id'=>$posted['variation_id']]
		);
		
		if($status){
			$myWishList = Wishlist::where(['user_id'=>Auth::user()->id,'sku_id'=>$posted['variation_id']])->first();
			return response()->json(['status' => 1, 'message' => "Product Added to Wish list",'data'=>['wish_list'=>$myWishList]]);
		} else {
			return response()->json(['status' => 0, 'message' => "Not Added."]);
		}
	}

	public function myWishList(){
		$data['cart_count'] = Cart::getCartCount();
		$data['my_wish_list'] = Wishlist::where(['user_id'=>Auth::user()->id])
		                           ->with(['variation.images'])
		                           ->whereHas('variation',function($query){
		                               $query->whereHas('currentBatch');
		                           })
		                           ->with(['variation'=>function($query2){
		                               $query2->with(['currentBatch']);
		                           }])
		                          ->get();
		return response()->json(['status' => 1, 'message' => 'My Wish List', 'data' => $data]);
	}
	
	public function removeFromMyWishList(Request $request)
	{
		$posted = $request->all();
		$validator = Validator::make($request->all(), [
                    'wishlist_id'=>'required|numeric'
        ]);
		$fields = array('wishlist_id');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
		
		$wishList = Wishlist::find($posted['wishlist_id']);
		
		if(!$wishList){
			$data['my_wish_list'] = Wishlist::where(['user_id'=>Auth::user()->id])->with(['variation.currentBatch','variation.images'])->get();
			return response()->json(['status' => 0, 'message' => 'Invalid Wishlist ID', 'data' => null]);
		}
		
		if($wishList->delete()){
			$data['my_wish_list'] = Wishlist::where(['user_id'=>Auth::user()->id])->with(['variation.currentBatch','variation.images'])->get();
			return response()->json(['status' => 1, 'message' => 'Successfully Removed from Wish List', 'data' => null]);
		} else {
			$data['my_wish_list'] = Wishlist::where(['user_id'=>Auth::user()->id])->with(['variation.currentBatch','variation.images'])->get();
			return response()->json(['status' => 0, 'message' => 'Not Successful.', 'data' => null]);
		}
	}
	
	
	/*
	//sample dummy method for getting filters options
	public function getFilters(){
		//$data['filters'] = ProductSpecification::whereNull('parent_id' )->with('filters')->get();		
		
		$fdata = DB::table('products')
            ->join('product_specifications', 'products.id', '=', 'product_specifications.product_id')
			->where(["products.category_id"=>'11','product_specifications.use_in_filters'=>'Yes'])->whereNotNull('product_specifications.parent_id')
			->select(DB::raw("product_specifications.name, GROUP_CONCAT(product_specifications.value SEPARATOR '|') AS filter_options"))
			->groupBy('product_specifications.name')
            ->get();
		foreach($fdata as $k=>$fdataOption){
			$fdata[$k]->filter_options = explode('|',$fdataOption->filter_options);
		}
	
		$data['filters'] = $fdata;
		
		return response()->json(['status' => 1, 'message' => 'Filter Data', 'data' => $data]);
	}
	*/
	
	

	/*API Method Ends here*/	
	
	
	/*Validation for Batch Number Unique*/
	public function validateProductBatchNumber(Request $request){
		
		$batches = Batch::whereBatchNumber($request->batch['batch_number'])->first();
		
		$isAvailable = true;
		if(!empty($batches)){
			$isAvailable = false;
		}
		//$isAvailable = false;
		echo json_encode(array(
		    'valid' => $isAvailable,
		));
	}
	/*Batch Number Validate ends*/
	
	
	/*Method used by other methods of the class itself*/
	public function getFilterOptions($cat_id) {
		
		$fdata = DB::table('product_skus')
			->join('products', 'products.id', '=', 'product_skus.product_id')
            ->join('product_specifications', 'product_skus.id', '=', 'product_specifications.sku_id')
			->join('category_product','products.id','=','category_product.product_id')
			->where(["category_product.category_id"=>$cat_id,'product_specifications.use_in_filters'=>'Yes'])->whereNotNull('product_specifications.parent_id')
			->select(DB::raw("product_specifications.name, GROUP_CONCAT(product_specifications.value SEPARATOR '|') AS filter_options"))
			->groupBy('product_specifications.name')
            ->get();
		 ;
		foreach($fdata as $k=>$fdataOption){
			$fdata[$k]->filter_options = explode('|',$fdataOption->filter_options);
		}
	
		return $fdata;
	}
	
}
