<?php

namespace App\Http\Controllers;


use Illuminate\Http\Request;
use Validator;
use App\Sku;
use App\Product;
use App\Category;
use App\Section;
use App\Banner;
use App\BannerProduct;
use DB;
use App\Cart;
use Auth;


class SectionsController extends Controller
{
    public function __construct()
    {
        //$this->middleware('auth');
    }


	public function create(){
		
		$data['categories'] =  Category::whereNull('parent_id')->where(['enabled'=>'Yes'])->with('categories')->orderBy('sort_order','asc')->get();
		
		return view('app_dashboard_sections/create',$data);
	}
	
	public function store(Request $request){
		//dd($request->all());
		
		$section = new Section;
		$section->name = $request->name;
		$section->enabled = $request->enabled;
		
		DB::beginTransaction();
		try {
			//getting sort order
			$sorder = Section::max('sort_order');
			$section->sort_order = $sorder+1;
			
			$section->save();
			
			$skus = [];
			foreach($request->sku_id as $key=>$skuId){
				$skus[$skuId]=['sort_order'=>$key];
			}
			
			$section->variations()->attach($skus);
			
			DB::commit();
			return redirect()->route('appDashboardSections')->with('success','Section Successfully Created.');
		} catch(\Exception $e){
			DB::rollback();
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
	}
	
	public function edit($id){
		
		$data['categories'] =  Category::whereNull('parent_id')->where(['enabled'=>'Yes'])->with('categories')->orderBy('sort_order','asc')->get();
		$data['section'] = Section::where('id',$id)->with('variations.images')->first();
		return view('app_dashboard_sections/edit',$data);
	}
	
	public function update(Request $request,$id){
		//dd($request->all());
		
		$section = Section::find($id);
		if(empty($section)){
			return redirect()->back()->with('error','Sorry! Invalid Section Details. Try again.');
		}
		
		$section->name = $request->name;
		$section->enabled = $request->enabled;
		
		DB::beginTransaction();
		try {
			
			$section->save();
			if($section->default=='No'){
				$section->variations()->detach();
				
				$skus = [];
				foreach($request->sku_id as $key=>$skuId){
					$skus[$skuId]=['sort_order'=>$key];
				}
				
				$section->variations()->attach($skus);
			}
			DB::commit();
			return redirect()->route('appDashboardSections')->with('success','Section Successfully Updated.');
		} catch(\Exception $e){
			DB::rollback();
			dd($e);
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
	}


    public function appDashboardSections(){
		
		$data['sections'] = Section::orderBy('sort_order','asc')->get();
		return view('app_dashboard_sections/appDashboardSections',$data);
	}
	
	public function updateSectionOrderAjax(Request $request){
		/*
		foreach($request->data as $key=>$secId){
			$section = Section::find($secId);
			$section->sort_order = $key;
			$section->save();
			
			$flag++;
		}
		*/
		DB::beginTransaction();
		try {
			foreach($request->data as $key=>$secId){
				Section::where('id',$secId)->update(['sort_order'=>$key]);
			}
			
			
			DB::commit();
			echo 1;
		} catch(\Exception $e){
			DB::rollback();
			echo 0;
		}
	}
	
	
	public function toggleSectionEnable($id,$enabled){
		if(Section::where('id',$id)->update(['enabled'=>$enabled])){
			return redirect()->back()->with('success','Updated Successfully.');
		} else {
			return redirect()->back()->with('error','Not updated. Try again.');
		}
	}
	
	public function destroy(Request $request,$id){
		DB::beginTransaction();
		$section = Section::find($id);
		
		if($section->default=='Yes'){
			return redirect()->back()->with('error','This Section can not be Deleted.');
		}
		
		try {
			$section = Section::find($id);
			$section->variations()->detach();
			$section->delete();
			
			DB::commit();
			return redirect()->back()->with('success','Section Successfully Deleted.');
		} catch(\Exception $e){
			DB::rollback();
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
		
		if(Section::where('id',$id)->delete()){
			return redirect()->back()->with('success','Deleted Successfully.');
		} else {
			return redirect()->back()->with('error','Not deleted. Try again.');
		}
	}
	
	
	public function getCategoryProductsAjax(Request $request){
		
		try{
			$catid = $request->catid;
			$products = Product::where('enabled','Yes')->with(['categories','variations.images'])
					->whereHas('categories',function($query)use($catid){
			    		$query->where('category_id',$catid);
			    	})->get();
					
			echo '<option value="">-- Select Products --</option>;';
			foreach($products as $product){
				echo "<optgroup label='".$product->name."'>";
				foreach($product->variations as $variation){
					echo "<option class='skoption".$variation->id."' value='".$variation->id."~".$product->product_code."~".$product->name."~".$variation->sku."~".$variation->variation_name."~".$variation->variation_value."~".$variation->images[0]->name."'>";
					echo $variation->sku." - ".$variation->variation_name." - ".$variation->variation_value;
					echo "</option>";
				}
				echo "</optgroup>";
			}
		} catch(\Exception $e){
			echo 0;
		}
	}


	public function slidersection(){
		$data['banners'] =  Banner::orderBy('sort_order','asc')->get();
		
		return view('app_dashboard_sections/slider',$data);
	}
	
	public function createsliderbanner(){
		
		$data['categories'] =  Category::whereNull('parent_id')->where(['enabled'=>'Yes'])->with('categories')->orderBy('sort_order','asc')->get();
		
		return view('app_dashboard_sections/create_banner',$data);
	}
	
	public function savebanner(Request $request){
		
		DB::beginTransaction();
		try {
			$banner = new Banner;	
			$banner->enabled = $request->enabled;
			
			//uploading image file
			$rFile = $request->file('name');
	        $rFileName = str_replace(" ","",str_replace(".","",microtime())) . 'banner.' . $rFile->getClientOriginalExtension();
	        $rFileUrl="https://gadgetzone.s3-ap-south-1.amazonaws.com/slider/".$rFileName;
	        $s3 = \Storage::disk('s3');
			$filePath = '/slider/' . $rFileName;
	        $imageResult=$s3->put($filePath, file_get_contents($rFile), 'public');
			if($imageResult==1){
				$banner->name = $rFileUrl;
			} else {
				return redirect()->back()->with('error','Sorry! Something went wrong while uploading image. Try again.');
			} 
			
			//getting sort order
			$sorder = Banner::max('sort_order');
			$banner->sort_order = $sorder+1;
			
			$banner->save();
			
			$skus = [];
			foreach($request->sku_id as $key=>$skuId){
				$skus[$skuId]=['sort_order'=>$key];
			}
			
			$banner->variations()->attach($skus);
			
			DB::commit();
			return redirect()->route('sections.slider')->with('success','Banner Successfully Uploaded.');
		} catch(\Exception $e){
			DB::rollback();
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
	}

	public function editsliderbanner($id){
		
		$data['banner'] = Banner::where('id',$id)->with('variations')->first();
		$data['categories'] =  Category::whereNull('parent_id')->where(['enabled'=>'Yes'])->with('categories')->orderBy('sort_order','asc')->get();
		
		
		
		return view('app_dashboard_sections/edit_banner',$data);
	}
	
	public function updatesliderbanner(Request $request){
		
		DB::beginTransaction();
		try {
			$id = $request->id;
			
			$banner = Banner::find($id);	
			$banner->enabled = $request->enabled;
			
			if($request->hasFile('name')){
				//uploading image file
				$rFile = $request->file('name');
		        $rFileName = str_replace(" ","",str_replace(".","",microtime())) . 'banner.' . $rFile->getClientOriginalExtension();
		        $rFileUrl="https://gadgetzone.s3-ap-south-1.amazonaws.com/slider/".$rFileName;
		        $s3 = \Storage::disk('s3');
				$filePath = '/slider/' . $rFileName;
		        $imageResult=$s3->put($filePath, file_get_contents($rFile), 'public');
				if($imageResult==1){
					$banner->name = $rFileUrl;
				} else {
					return redirect()->back()->with('error','Sorry! Something went wrong while uploading image. Try again.');
				} 
			}
			
			
			$banner->save();
			$banner->variations()->detach();
			
			$skus = [];
			foreach($request->sku_id as $key=>$skuId){
				$skus[$skuId]=['sort_order'=>$key];
			}
			
			$banner->variations()->attach($skus);
			
			DB::commit();
			return redirect()->route('sections.slider')->with('success','Banner Successfully Updated.');
		} catch(\Exception $e){
			DB::rollback();
			
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
	}

	public function destroyBanner(Request $request){
		DB::beginTransaction();
		try {
			$banner = Banner::find($request->id);
			$banner->variations()->detach();
			$banner->delete();
			
			DB::commit();
			return redirect()->back()->with('success','Banner Successfully Deleted.');
		} catch(\Exception $e){
			DB::rollback();
			return redirect()->back()->with('error','Sorry! Something went wrong. Try again.');
		}
		
		
		
		if(Section::where('id',$id)->delete()){
			return redirect()->back()->with('success','Deleted Successfully.');
		} else {
			return redirect()->back()->with('error','Not deleted. Try again.');
		}
	}


	public function updateBannerOrderAjax(Request $request){
		DB::beginTransaction();
		try {
			foreach($request->data as $key=>$secId){
				Banner::where('id',$secId)->update(['sort_order'=>$key]);
			}
			
			DB::commit();
			echo 1;
		} catch(\Exception $e){
			DB::rollback();
			echo 0;
		}
	}
	
	
	public function toggleBannerEnable($id,$enabled){
		if(Banner::where('id',$id)->update(['enabled'=>$enabled])){
			return redirect()->back()->with('success','Updated Successfully.');
		} else {
			return redirect()->back()->with('error','Not updated. Try again.');
		}
	}
	
	public function getBannerProductsAjax(Request $request){
		$data['banner'] = Banner::where('id',$request->id)->with('variations')->first();
		
		return view('app_dashboard_sections/getBannerProducts',$data);
	}
	
	public function updateBannerProductOrderAjax(Request $request){
		DB::beginTransaction();
		try {
			foreach($request->data as $key=>$secId){
				$ids = explode('~',$secId);
				
				BannerProduct::where(['banner_id'=>$ids[0],'sku_id'=>$ids[1]])->update(['sort_order'=>$key]);
			}
			
			DB::commit();
			echo 1;
		} catch(\Exception $e){
			DB::rollback();
			dd($e);
			echo 0;
		}
	}
	
	
	/*
	 * API methods
	 */
	 
	 public function appDashboard(Request $request){
		
		$sectionOrder = Section::whereEnabled('Yes')->with('variations')->orderBy('sort_order','asc')->get();
		
		$categories = Category::where('name','!=','Uncategorized')->where(function($query){
			$query->where('parent_id','0')
				  ->orWhere('parent_id','')
				  ->orWhereNull('parent_id');
		})->where('enabled','Yes')->orderBy('sort_order','asc')->get();
		$data['categories'] = $categories;
		$data['cart_count'] = $this->getCartCount();
		
		$data['sections'] = [];
		
		foreach($sectionOrder as $sOrder){
			$sOrder = $sOrder->toArray();
			switch ($sOrder['app_key']) {
			    case 'slider':
			       $sOrder['banners'] = Banner::whereEnabled('Yes')->orderBy('sort_order','asc')->get();
				   unset($sOrder['variations']);
			        break;
			    case 'popular':
			       $sOrder['variations'] = Sku::with(['product','currentBatch','images','productSpecifications','cart','wishList'])->whereHas('product',function($query){
												$query->where(['popular'=>'Yes','enabled'=>'Yes'])->with('categories')->whereHas('categories',function($query){
													$query->where('name','!=','Uncategorized');
												});
											})->take('6')->get()->toArray();
			        break;
			    case 'recent':
			        $sOrder['variations'] = Sku::with(['product','currentBatch','images','productSpecifications','cart','wishList'])->whereHas('product',function($query){
												$query->where(['popular'=>'Yes','enabled'=>'Yes'])->with('categories')->whereHas('categories',function($query){
													$query->where('name','!=','Uncategorized');
												});
											})->take('6')->get()->toArray();
			        break;
			   default:
			         break;
			}
			
			$data['sections'][] = $sOrder;
			
		}
		
		
	 	return response()->json(['status' => 1, 'message' => 'Application Dashboard', 'data' => $data]);
	}


	public function sectionProducts(Request $request){
		
		$posted = $request->all();
		$validator = Validator::make($request->all(), [
                    'section_id'=>'required|numeric'
        ]);
		$fields = array('section_id');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
		
		
		$section = Section::find($request->section_id);
		if(empty($section)){
			return response()->json(['status' => 0, 'message' => "Invalid Section ID"]);
		}
		
		$products = $section->variations()->paginate(25);
		
		$data['products'] = $products;
		$data['cart_count'] = $this->getCartCount();
		
		return response()->json(['status' => 1, 'message' => "Section Products",'data'=>$data]);
		
	}
	
	public function bannerProducts(Request $request){
		
		$posted = $request->all();
		$validator = Validator::make($request->all(), [
                    'banner_id'=>'required|numeric'
        ]);
		$fields = array('banner_id');
		
        $error_message = "";
        if ($validator->fails()) {
            foreach ($fields as $field) {
                if (isset($validator->errors()->getMessages()[$field][0]) && !empty($validator->errors()->getMessages()[$field][0])) {
                    $error_message .= $validator->errors()->getMessages()[$field][0].", ";
                }
            }
            return response()->json(['status' => 0, 'message' => substr($error_message,0,-2)]);
        }
		
		
		$banner = Banner::find($request->banner_id);
		if(empty($banner)){
			return response()->json(['status' => 0, 'message' => "Invalid Banner ID"]);
		}
		
		$products = $banner->variations()->paginate(25);
		$data['products'] = $products;
		$data['cart_count'] = $this->getCartCount();
		return response()->json(['status' => 1, 'message' => "Banner Products",'data'=>$data]);
		
	}


	public function getCartCount(){
		$count = 0;
		if(!empty(Auth::user())){
			$count = Cart::where('user_id',Auth::user()->id)->count();
		}
		return $count;
	}
	 
	 
	 
	
	
	
}
