<?php
namespace App\Http\Controllers;

use App\Address;
use App\Category;
use App\Culture;
use App\DeliveryAddress;
use App\Product;
use App\Seller;
use App\SellerCulture;
use App\SellerDeliveryAddress;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;
use App\Cart;
use Illuminate\Support\Facades\Auth;
use Facade\FlareClient\Http\Response;
use App\Sku;
use App\NotDeliveryAddress;
use App\Settings;
use App\Brand;
use App\Mail\OrderMail;
use Illuminate\Support\Facades\Mail;
use App\UserForm;
use App\OrderProduct;
class SellersController extends Controller
{

    public $User;

    public function __construct()
    {
        // $this->middleware('auth');
        $this->User = new Seller();
    }

    public function sellersList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['users'] = User::where($request->ftype, 'like', '%' . $request->stext . '%')->where('role', 'Seller')->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['users'] = User::where('role', 'Seller')->paginate(400);
            $data['ftype'] = "";
            $data['stext'] = "";
        }

        return view('seller/sellersList', $data);
    }

    public function addSeller()
    {
        $addresses = DeliveryAddress::take(50)->get();
        $cultures = Culture::where('enabled', 'Yes')->get();
        return view('seller/addSeller', [
            'addresses' => $addresses,
            'cultures' => $cultures
        ]);
    }
    
    public function newDashboard()
    {
        return view('new-dashboard/home');
    }

    public function saveSeller(Request $request)
    {
        $posted = collect($request->all())->toArray();

        // echo "<pre>";print_r($posted);exit;

        $request->validate([
            'email' => 'unique:users',
            'phone' => 'unique:users'
        ], [
            'email.unique' => 'Seller already exists with this email',
            'phone.unique' => 'Seller with this phone number already exists.'
        ]);

        $user = new User();
        $user->name = $posted['name'];
        if (! empty($posted['phone'])) {
            $user->country_code = "+" . $posted['country_code'];
            $user->phone = $posted['phone'];
        }
        $user->email = $posted['email'];
        $user->password = Hash::make($posted['password']);
        $user->enabled = $posted['enabled'];
        $user->role = "Seller";

        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $user->photo = $imageUrl;
            }
        }

        if ($user->save()) {

            $cultures = $request->cultures;
            if (is_array($cultures)) {
                foreach ($cultures as $culture) {
                    // echo $address;exit;
                    $sellerCulture = new SellerCulture();
                    $sellerCulture->seller_id = $user->id;
                    $sellerCulture->culture_id = $culture;
                    $sellerCulture->save();
                }
            }

            return redirect()->route('sellerProfile', $user->id)->with('success', 'Seller created successfully.');
        } else {
            return back()->with('error', 'Seller has not been created.');
        }
    }

    public function sellerProfile($id = null)
    {
        $user = User::where([
            'id' => $id
        ])->where('role', 'Seller')
            ->with('deliveryAddresses')
            ->with('sellerCultures')
            ->first();

        // echo"<pre>";print_r($user);exit;

        if (! $user) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller Details');
        }

        if (! User::isSuperAdmin() && ! User::isOwnAccount($id)) {
            return back()->with('error', 'You are not allowed to access this page.');
        }

        if (User::isSeller()) {
            return view('seller/seller/sellerProfile', [
                'user' => $user
            ]);
        }
        return view('seller/sellerProfile', [
            'user' => $user
        ]);
    }
    
    
    
    public function sellerProducts(Request $request,$id)
    {
        $data['posted'] = [];
        
        $data['user'] = User::where(['id' => $id])->where('role', 'Seller')->first();
        
        if (empty($data['user'])) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller Details');
        }
        
        if (! User::isSuperAdmin() && ! User::isOwnAccount($id)) {
            return back()->with('error', 'You are not allowed to access this page.');
        }
        
        $skus = Sku::whereHas('product',function ($query) use ($id) {
                        $query->whereHas('variations', function ($query2) use ($id) {
                            $query2->whereHas('currentBatch', function ($query3) use ($id) {
                                $query3->where('seller_id', $id);
                            });
                        });
                    })
                    ->with(['product'=>function ($query) use ($id) {
                        $query->with(['variations'=> function ($query2) use ($id) {
                            $query2->with(['currentBatch'=> function ($query3) use ($id) {
                                $query3->where('seller_id', $id);
                            }]);
                        }]);
                    }])
                    ->with(['product.categories','images','productSpecifications','batches'])->orderBy('id','desc');
        
        
        $data['products'] = $skus->paginate(30);
        
        
                                
        $data['categories'] = Category::whereNull('parent_id')->with(['categories'=>function($query){
            $query->where('enabled','Yes');
        }])->orderBy('sort_order','asc')->get()->toArray();
                            
       // echo"<pre>";print_r( $data['products']);exit;
        
        return view('seller/sellerProducts', [
            'data' => $data
        ]);
    }
    
    public function sellerSkudetails($sku_id,$sellerId)
    {
        $user = User::where(['id' => $sellerId])->where('role', 'Seller')->first();
        
        if (empty($user)) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller Details');
        }
        
        $sku = Sku::
                with(['images','productSpecifications','product.categories','batches.invoice.merchant','currentBatch'])
                ->find($sku_id);
        
        
        if(empty($sku)){
            return redirect()->route('sellersList')->with('warning','Invalid SKU details. Kindly check and try again.');
        }
        $data=[
            'sku'=>$sku,
            'sellerId' => $sellerId,
            'seller' => $user
        ];
//         print_r($sku->toArray());exit;
        return view('seller/sellerSkudetails',$data);
    }

    public function editSeller($id)
    {
        $user = User::where([
            'id' => $id
        ])->with('deliveryAddresses')
            ->with('sellerCultures')
            ->where('role', 'seller')
            ->first();

        if (! $user) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller to edit');
        }

        if (! User::isSuperAdmin() && ! User::isOwnAccount($id)) {
            return back()->with('error', 'You are not allowed to access this page.');
        }

        $selectedAddress = [];
        foreach ($user->deliveryAddresses as $address) {
            $selectedAddress[] = $address->id;
        }

        $selectedCultures = [];
        foreach ($user->sellerCultures as $culture) {
            $selectedCultures[] = $culture->id;
        }

        $addresses = DeliveryAddress::take(50)->get();
        $cultures = Culture::where('enabled', 'Yes')->get();

        if (User::isSeller()) {
            return view('seller/seller/editSeller', [
                'user' => $user,
                'addresses' => $addresses,
                'cultures' => $cultures,
                'selectedAddress' => $selectedAddress,
                'selectedCultures' => $selectedCultures
            ]);
        }

        return view('seller/editSeller', [
            'user' => $user,
            'addresses' => $addresses,
            'cultures' => $cultures,
            'selectedAddress' => $selectedAddress,
            'selectedCultures' => $selectedCultures
        ]);
    }

    public function saveEditSeller(Request $request, $id)
    {
//         echo "<pre>";print_r($request->all());exit;
        $posted = collect($request->all())->toArray();

        $request->validate([
            'email' => 'unique:users,id,' . $id,
            'phone' => 'unique:users,id,' . $id
        ], [
            'email.unique' => 'User already exists with this email',
            'phone.unique' => 'User with this Phone already exists.'
        ]);

        $user = User::where([
            'id' => $id
        ])->where('role', 'Seller')->first();
        if (! $user) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller to edit');
        }
        $user->name = $posted['name'];
        $user->country_code = "+" . $posted['country_code'];
        $user->phone = $posted['phone'];
        $user->email = $posted['email'];
        if (! empty($user->password)) {
            $user->password = Hash::make($posted['password']);
        }
        $user->enabled = $posted['enabled'];

        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $user->photo = $imageUrl;
            }
        }

        if ($user->save()) 
        {
            $postcodeDetails = $request->postcodes;
            
            if(!empty($postcodeDetails))
            {
                //Save delivery address if not saved
                foreach ($postcodeDetails as $postcodeDetail)
                {
                    $splitData = explode('~', $postcodeDetail);
                    $postcode = $splitData[0];
                    $lat = $splitData[1];
                    $long = $splitData[2];
                    
                    
                    $deliveryAddress = DeliveryAddress::where('Postcode',$postcode)->first();
                    if(empty($deliveryAddress))
                    {
                        $deliveryAddressObj = new DeliveryAddress();   
                        $deliveryAddressObj->Postcode = $postcode;
                        $deliveryAddressObj->Latitude = $lat;
                        $deliveryAddressObj->Longitude = $long;
                        $deliveryAddressObj->save();
                    }
                }
               
                //Save delivery address in seller delivery list
                foreach ($postcodeDetails as $postcodeDetail)
                {
                    $splitData = explode('~', $postcodeDetail);
                    $postcode = $splitData[0];
                    
                    $deliveryAddress = DeliveryAddress::where('Postcode',$postcode)->first();
                    if(!empty($deliveryAddress))
                    {
                        $alreadySaved = SellerDeliveryAddress::where(['seller_id'=>$user->id,'delivery_address_id'=>$deliveryAddress->id])->first();
                        if(empty($alreadySaved))
                        {
                            $sellerDeliveryAddress = new SellerDeliveryAddress();
                            $sellerDeliveryAddress->seller_id = $user->id;
                            $sellerDeliveryAddress->delivery_address_id = $deliveryAddress->id;
                            $sellerDeliveryAddress->save();
                        }
                    }
                }
            }
            
            $cultures = $request->cultures;
            if (is_array($cultures)) {
                $user->sellerCultures()->detach();
                foreach ($cultures as $culture) {
                    // echo $address;exit;
                    $sellerCulture = new SellerCulture();
                    $sellerCulture->seller_id = $user->id;
                    $sellerCulture->culture_id = $culture;
                    $sellerCulture->save();
                }
            }
            return redirect()->route('sellerProfile', $user->id)->with('success', 'Seller updated successfully.');
        } else {
            return back()->with('error', 'Seller has not been modified.');
        }
    }

    public function editSellerAddress($id)
    {
        $user = User::where([
            'id' => $id
        ])->with('addresses')
            ->where('role', 'seller')
            ->first();

        if (! $user) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller to edit');
        }

        if (! User::isSuperAdmin() && ! User::isOwnAccount($id)) {
            return back()->with('error', 'You are not allowed to access this page.');
        }

        if (User::isSeller()) {
            return view('seller/seller/editSellerAddress', [
                'user' => $user
            ]);
        }

        return view('seller/editSellerAddress', [
            'user' => $user
        ]);
    }

    public function saveEditSellerAddress(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();

        $user = User::where([
            'id' => $id
        ])->where('role', 'Seller')->first();
        if (! $user) {
            return redirect()->route('sellersList')->with('warning', 'Invalid Seller to edit');
        }

        $address = Address::where('user_id', $user->id)->first();

        if (empty($address)) {
            $address = new Address();
            $address->user_id = $user->id;
            $address->user_type = Address::USER_TYPE_SELLER;
        }

        if (empty($posted['postcode'])) {
            return back()->with('error', 'Postcode is required.');
        }
        
        if (empty($request->lat) || empty($request->long)) {
            return back()->with('error', 'Postcode is invaild.');
        }

        if(!empty($request->postcode)){
            $address->zip = $posted['postcode'];
            $address->longitude = $posted['long'];
            $address->latitude = $posted['lat'];
        }
        
        $address->address1 = $posted['address1'];
        $address->address2 = $posted['address2'];
        $address->landmark = $posted['landmark'];
        $address->city = $posted['city'];
        $address->state = $posted['state'];
        $address->country = $posted['country'];
        $address->open_time = $posted['open_time'];
        $address->close_time = $posted['close_time'];
        $address->default = 'Yes';

        if ($address->save()) {
            return redirect()->route('sellerProfile', $user->id)->with('success', 'Seller address updated successfully.');
        } else {
            return back()->with('error', 'Seller has not been modified.');
        }
    }
    
    public function changeStoreOpenStatus(Request $request)
    {

        $store = User::find(Auth::user()->id);
        if($store->role != 'Seller'){
            return response()->json([
                'status' => 0
            ]);
        }

        if($request['is_open'] ==1){
            $store['is_open'] = 'Yes';


        }else{
            
            // Mail::send('mail.storeClosed', $store, function($message) {
            //      $message->to('abc@gmail.com', 'Tutorials Point')->subject
            //         ('Store has been Closed');
            //      $message->from('xyz@gmail.com','Virat Gandhi');
            //   });
            $store['is_open'] = 'No';
        }
        
        // $store->is_open = ($request->is_open == 'true') ? "Yes" : "No";
        if($store->save()){
            return response()->json([
                'status' => 1
            ]);
        }
        return response()->json([
            'status' => 0
        ]);
    }
    
    // Function to mark a store as coming soon or available in app
    public function markStoreExist(Request $request,$id,$popular){
        //dd($request->all());
        
        User::where('id',$id)->update(['store_exist'=>$popular]);
        
        return redirect()->back()->with('success','Updated');
    }
    
    // Function to mark a store as enabled or disabled
    public function markEnabled(Request $request,$id,$popular){
        //dd($request->all());
        
        User::where('id',$id)->update(['enabled'=>$popular]);
        
        return redirect()->back()->with('success','Updated');
    }

    /**
     * API start here
     */

    /*
     * Return list of sellers(stores) based on postcode
     */
    public function getSellerStoresList(Request $request)
    {
        $latitude = $longitude = "";
        if (! empty($request->postcode))
        {
            $postCode = NotDeliveryAddress::isValidPostcode($request->postcode);
           
            if (!empty($postCode)) {
                return response()->json([
                    'status' => 0,
                    'message' => "We are not delivered in this area."
                ]);
            }
        }
        
        if (! empty($request->Latitude) && ! empty($request->Longitude)) {

            $latitude = $request->Latitude;
            $longitude = $request->Longitude;
        } 

        if (empty($latitude) || empty($longitude)) {
            return response()->json([
                'status' => 0,
                'message' => "Latitude and Longitude are required."
            ]);
        }
        
        $sellerIds = Address::getSellerBasedOnDistance($latitude, $longitude);
        
//         print_r($postcodes);exit;
        
        if(!$sellerIds)
        {
            return response()->json([
                'status' => 0,
                'message' => "No store found in youe area."
            ]);
        }

        $sellers = User::where('role', 'Seller')->with([
                        'addresses',
                        'sellerCultures'
                    ])
                    ->whereIn('id', $sellerIds)
                    ->orderBy('sort_order','ASC');
        

        if (! empty($request->culture_id)) {
            $cultureId = $request->culture_id;
            $sellers = $sellers->whereHas('sellerCultures', function ($query) use ($cultureId) {
                $query->where('culture_id', $cultureId);
            });
        }
        
        if ($request->sort == 'distance') {
            
            $maxDistance = Settings::getMaxDistanceInMiles();                        
            
            $sellers = User::select(DB::raw('users.*,addresses.user_id,
                            ( 6367 * acos( cos( radians(' . $latitude . ') )
                            * cos( radians( latitude ) ) * cos( radians( longitude ) -
                            radians(' . $longitude . ') ) + sin( radians(' . $latitude . ') )
                            * sin( radians( latitude ) ) ) ) AS distance'))
                            ->whereRaw('( 6367 * acos( cos( radians(' . $latitude . ') )
                            * cos( radians( latitude ) ) * cos( radians( longitude ) -
                            radians(' . $longitude . ') ) + sin( radians(' . $latitude . ') )
                            * sin( radians( latitude ) ) ) )' . '< ?', $maxDistance)
                            ->with(['addresses','sellerCultures'])
                            ->join('addresses','users.id','=','addresses.user_id')
                            ->orderBy('distance')
                            ->where('role','Seller');
                            
            
            if (! empty($request->culture_id)) {
                $cultureId = $request->culture_id;
                $sellers = $sellers->whereHas('sellerCultures', function ($query) use ($cultureId) {
                    $query->where('culture_id', $cultureId);
                });
            }
        }
        
        if($request->sort == 'top_rated')
        {
            $sellers =   User::select('users.*')->where('role', 'Seller')
                        ->with(['addresses','sellerCultures'])
                        ->whereIn('users.id',$sellerIds)
                        ->join('ratings', 'users.id', '=', 'ratings.seller_id')
                        ->groupBy('users.id')
                        ->orderByRaw('AVG(ratings.rating) DESC');
                        
            if (! empty($request->culture_id)) {
                $cultureId = $request->culture_id;
                $sellers = $sellers->whereHas('sellerCultures', function ($query) use ($cultureId) {
                    $query->where('culture_id', $cultureId);
                });
            }
        }
        
        $sellers = $sellers->paginate(50);
//         foreach ($sellers as $seller)
//         {
//             $time = $seller->getStoreDistance($latitude,$longitude);
//             $seller['travel_min'] = $time ;
//         }
        

        if (count($sellers) == 0) {
            return response()->json([
                'status' => 0,
                'message' => "No seller store found on this postcode.",
                'seller_found' => "no"
            ]);
        }

        return response()->json([
            'status' => 1,
            'message' => "Stores send successfully.",
            'data' => $sellers,
            'cart_count' => Cart::getCartCount(),
            'seller_found' => "yes"
        ]);
    }

    /*
     * return list of all sellers
     */
    public function getAllSellerStoresList(Request $request)
    {
//         if (empty($request->latitude) || empty($request->longitude)) {
//             return response()->json([
//                 'status' => 0,
//                 'message' => "Latitude and Longitude are required."
//             ]);
//         }

        if($request->show_coming_soon == 'no')
        {
            $stores = User::where('enabled', 'Yes')->where('role', 'Seller')->with(['addresses','sellerCultures'])
            ->where('store_exist','Yes')
            ->orderBy('sort_order','ASC');
        }else
        {
            $stores = User::where('enabled', 'Yes')->where('role', 'Seller')->with(['addresses','sellerCultures'])
            ->orderBy('sort_order','ASC');
        }
        
       
        
        if (! empty($request->culture_id)) {
            $cultureId = $request->culture_id;
            $stores = $stores->whereHas('sellerCultures', function ($query) use ($cultureId) {
                $query->where('culture_id', $cultureId);
            });
        }

        if ($request->sort == 'distance') {
            if (empty($request->latitude) || empty($request->longitude)) {
                return response()->json([
                    'status' => 0,
                    'message' => "Latitude and Longitude are required on distance sort."
                ]);
            }
            
            $latitude = $request->latitude;
            $longitude = $request->longitude;

            $stores =  User::select(DB::raw('users.*,addresses.user_id,MIN(
                        ( 6367 * acos( cos( radians(' . $latitude . ') )
                        * cos( radians( latitude ) ) * cos( radians( longitude ) -
                        radians(' . $longitude . ') ) + sin( radians(' . $latitude . ') )
                        * sin( radians( latitude ) ) ) )) AS distance'))
                        ->with(['addresses','sellerCultures'])
                        ->join('addresses','users.id','=','addresses.user_id')
                        ->orderBy('distance')
                        ->where('role', 'Seller')
//                         ->where('store_exist','Yes')
                        ->groupBy('users.id');
                        
            
            if (! empty($request->culture_id)) {
                $cultureId = $request->culture_id;
                $stores = $stores->whereHas('sellerCultures', function ($query) use ($cultureId) {
                    $query->where('culture_id', $cultureId);
                });
            }
        }
        
        if($request->sort == 'top_rated')
        {   
            $stores = User::select('users.*')->where('role', 'Seller')
                        ->with(['addresses','sellerCultures'])
                        ->join('ratings', 'users.id', '=', 'ratings.seller_id')
                        ->groupBy('users.id')
//                         ->where('store_exist','Yes')
                        ->orderByRaw('AVG(ratings.rating) DESC');
            
            
            if (! empty($request->culture_id)) {
                $cultureId = $request->culture_id;
                $stores = $stores->whereHas('sellerCultures', function ($query) use ($cultureId) {
                    $query->where('culture_id', $cultureId);
                });
            }
        }
        
        $stores = $stores->paginate(20);
//         foreach ($stores as $seller)
//         {
//             $res = $seller->getStoreDistance($request->latitude,$request->longitude);
//             $seller['travel_min'] = $res['time'];
//             $seller['travel_miles'] = $res['miles'];
//         }

        if (count($stores) == 0) {
            return response()->json([
                'status' => 0,
                'message' => "Stores not found."
            ]);
        }
    

        return response()->json([
            'status' => 1,
            'message' => "Stores Send Successfully",
            'cart_count' => Cart::getCartCount(),
            'data' => $stores
        ]);
    }

    public function getSellerStoreDetail($id)
    {
//         $data['seller'] = User::with([
//             'addresses',
//             'sellerCultures'
//         ])->find($id);

//         if (empty($data['seller'])) {
//             return response()->json([
//                 'status' => 0,
//                 'message' => "Seller not found."
//             ]);
//         }

        $data['category'] = Category::whereNull('parent_id')
        ->whereHas('subCategories',function($query)use($id){
            $query->whereHas('products', function ($query1) use ($id) {
                $query1->whereHas('variations', function ($query2) use ($id) {
                    $query2->whereHas('currentBatch', function ($query3) use ($id) {
                        $query3->where('seller_id', $id);
                    });
                });
            });
                $query->where('name','!=','Uncategorized');
        })
        ->with(['subCategories'=>function($query)use($id){
            $query->whereHas('products', function ($query1) use ($id) {
                $query1->whereHas('variations', function ($query2) use ($id) {
                    $query2->whereHas('currentBatch', function ($query3) use ($id) {
                        $query3->where('seller_id', $id);
                    });
                });
            });
            $query->where('name','!=','Uncategorized');
        }])
        ->where('name','!=','Uncategorized')
        ->get();
            
        $data['all_category'] = "";
                                    
        $cart_count = Cart::getCartCount();

        return response()->json([
            'status' => 1,
            'message' => "Store Details",
            'data' => $data,
            'cart_count' => $cart_count
        ]);
    }
    
    public function popularProducts(Request $request)
    {
        if(empty($request->seller_id)){
            return response()->json([
                'status' => 0,
                'message' => "Store id is required."
            ]);
        }
        $id = $request->seller_id;
        
        $data['popular_products'] = Product::where('popular' , 'yes')->with([
                                        'brand','variations.currentBatch',
                                        'variations.images',
                                        'variations.cart','variations.wishList'
                                    ])->where('enabled','Yes')
                                    ->whereHas('variations', function ($query2) use ($id) {
                                        $query2->whereHas('currentBatch', function ($query3) use ($id) {
                                            $query3->where('seller_id', $id);
                                        });
                                    })
                                    ->orderBy('id','DESC')->take(10)->get();
                                    
        return response()->json([
            'status' => 1,
            'message' => "Popular Products",
            'data' => $data
        ]);
    }

    public function getListOfAllThings()
    {
        $data = [];

        $data['cultures'] = Culture::where('enabled', 'Yes')->orderBy('id', 'DESC')->take(5)->get();
        
        $data['brands'] = Brand::where('enabled', 'Yes')->orderBy('id', 'DESC')->take(5)->get();
        
        $data['products'] = Product::where('enabled', 'Yes')
                            ->whereHas('variations',function($query){
                                $query->whereHas('currentBatch',function($query2){
                                    $query2->whereHas('users',function($query3){
                                        $query3->where('store_exist','Yes');
                                    });
                                });
                            })
                            ->orderBy('id', 'ASC')->take(5)->get();
                            
        $data['stores'] = User::where('enabled', 'Yes')->with([
                            'addresses',
                            'sellerCultures'
                        ])->where('role', 'Seller')->where('store_exist','Yes')->orderBy('id', 'DESC')->take(5)->get();

        return response()->json([
            'status' => 1,
            'message' => "List of all things send successfully.",
            'data' => $data
        ]);
    }
    
    /*
     * return data from single table based on type
     */
    public function searchByCategory(Request $request)
    {
        $searchKeyword = $request->keyword;
        $searchType = $request->type;
        
        if(empty($searchKeyword) || empty($searchType)){
            return response()->json([
                'status' => 0,
                'message' => "keyword and type both are required."
            ]);
        }
        $data = [];
        if($searchType == "culture"){
            $data = Culture::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')->paginate(15);
        }
        if($searchType == "brand"){
            $data = Brand::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')->paginate(15);
        }
        if($searchType == "product"){
            $data = Product::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')
            ->with(['tax','categories','variations.currentBatch','variations.images'/*,'variations.productSpecifications'*/,'variations.cart','variations.wishList'])
                    ->whereHas('variations',function($query){
                        $query->whereHas('currentBatch');
                    })->paginate(15);
        }
        if($searchType == "store"){
            $data = User::where('enabled', 'Yes')->where('role', 'Seller')->where('name', 'LIKE', '%' . $searchKeyword . '%')->paginate(15);
        }
        
        if(count($data) == 0){
            return response()->json([
                'status' => 0,
                'message' => "No record found."
            ]);
        }
        
        return response()->json([
            'status' => 1,
            'message' => "data send successfully.",
            'data' => $data
        ]);
    }
    
    /*
     * return data from three table based on search keyword
     */
    public function searchAnything(Request $request)
    {
        $searchKeyword = $request->keyword;
        
        if(empty($searchKeyword)){
            return response()->json([
                'status' => 0,
                'message' => "keyword field is required."
            ]);
        }
        
        if(empty($request->seller_id))
        {
            if(empty($request->latitude) || empty($request->longitude))
            {
                return response()->json([
                    'status' => 0,
                    'message' => "Latitude and Longitude is required."
                ]);
            }
            $latitude = $request->latitude;
            $longitude = $request->longitude;
            
            $data['brands'] = Brand::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')->get();
            
            $data['cultures'] = Culture::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')->get();
            
            $sellerIds = Address::getSellerBasedOnDistance($latitude,$longitude);
            $data['products'] = [];
            $data['stores'] = [];
            
            //if sellerIds are not empty
            if($sellerIds)
            {
                $data['products'] = Product::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')
                                    ->with(['variations.currentBatch',
                                        'variations.images',
                                        'variations.cart','variations.wishList'
                                    ])
                                    ->whereHas('variations',function($query) use ($sellerIds){
                                        $query->whereHas('currentBatch',function($query2) use ($sellerIds){
                                            $query2->whereHas('users',function($query3) use ($sellerIds){
                                                $query3->where('store_exist','Yes');
                                                $query3->whereIn('id',$sellerIds);
                                            });
                                        });
                                    })
                                    ->get();
                
                
                $data['stores'] = User::where('enabled', 'Yes')->where('role', 'Seller')
                                ->where('store_exist','Yes')
                                ->where('name', 'LIKE', '%' . $searchKeyword . '%')
                                ->whereIn('id',$sellerIds)
                                ->get();
            }
            
        }else{
            
            $sellerId = $request->seller_id;
            
            $data['brands'] =  Brand::where('enabled', 'Yes')->where('name', 'LIKE', '%' . $searchKeyword . '%')
                                ->whereHas('products',function($query) use ($sellerId) {
                                    $query->whereHas('variations',function($query2) use ($sellerId){
                                        $query2->whereHas('currentBatch', function ($query3) use ($sellerId) {
                                            $query3->where('seller_id', $sellerId);
                                        });
                                    });
                                })
                                ->with(['products'=>function($query) use ($sellerId) {
                                    $query->whereHas('variations',function($query2) use ($sellerId){
                                        $query2->whereHas('currentBatch', function ($query3) use ($sellerId) {
                                            $query3->where('seller_id', $sellerId);
                                        });
                                    });
                                }])
                                ->get();
             
            $data['products'] = Product::where('enabled', 'Yes')
                                ->where('name', 'LIKE', '%' . $searchKeyword . '%')
                                ->with(['categories','variations.currentBatch','variations.currentBatch.country',
                                    'variations.images','tax',
                                    'variations.cart','variations.wishList'
                                ])
                                ->whereHas('variations',function($query) use ($sellerId){
                                    $query->whereHas('currentBatch', function ($query3) use ($sellerId) {
                                        $query3->where('seller_id', $sellerId);
                                    });
                                })->get();
                                
            
            $data['categories'] = Category::whereNull('parent_id')
                ->whereHas('subCategories',function($query)use($sellerId){
                    $query->whereHas('products', function ($query1) use ($sellerId) {
                        $query1->whereHas('variations', function ($query2) use ($sellerId) {
                            $query2->whereHas('currentBatch', function ($query3) use ($sellerId) {
                                $query3->where('seller_id', $sellerId);
                            });
                        });
                    });
                    $query->where('name','!=','Uncategorized');
                })
                ->with(['subCategories'=>function($query)use($sellerId){
                    $query->whereHas('products', function ($query1) use ($sellerId) {
                        $query1->whereHas('variations', function ($query2) use ($sellerId) {
                            $query2->whereHas('currentBatch', function ($query3) use ($sellerId) {
                                $query3->where('seller_id', $sellerId);
                            });
                        });
                    });
                    $query->where('name','!=','Uncategorized');
                }])
            ->where('name','LIKE', '%' . $searchKeyword . '%')
            ->get();
            
        }
        
        return response()->json([
            'status' => 1,
            'message' => "data send successfully.",
            'data' => $data
        ]);
    }
    
    public function checkDeliveryPostcode(Request $request)
    {
        if (empty($request->latitude) || empty($request->longitude) || empty($request->seller_id) || empty($request->postcode)) 
        {
            return response()->json([
                'status' => 0,
                'message' => "All 4 params are required."
            ]);
        }
                
        $notPostCode = NotDeliveryAddress::whereRaw('REPLACE (Postcode," ","") LIKE "%' . str_replace(' ', '%', $request->postcode) . '%"')->first();       
        if (!empty($notPostCode)) {
            return response()->json([
                'status' => 0,
                'message' => "We are not delivered in this area."
            ]);
        }
        
        $seller = User::where('role', 'Seller')->where('id',$request->seller_id)->first();
        
        $distance = $seller->getStoreDistanceFromUser($request->latitude, $request->longitude);
        
        $maxDistance = Settings::getMaxDistanceInMiles();  
                    
       if($distance > $maxDistance)
        {
            return response()->json([
                'status' => 0,
                'message' => "We do not deliver at this address, please change your postcode and try again." . $distance
            ]);
        }
        
        return response()->json([
            'status' => 1,
            'd' => $maxDistance
        ]);
    }  

//     public function bestSellerByProduct(){
//         $bestSellerByProduct = OrderProduct::with('productSkus')->groupBy('sku_id')->orderByRaw('COUNT(*) DESC')->first();
//         dd($bestSellerByProduct);
// //         $skuData =  OrderProduct::find($bestSellerByProductId)->getSkuData;
// //         dd($skuData);
// // where('id', )
//     }
}
