<?php

namespace App\Http\Controllers;
use App\Specification;
use Illuminate\Http\Request;
use Validator;
use DB;

class SpecificationsController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
    	$specifications = Specification::orderBy('id','asc')->orderBy('parent_id','asc')->orderBy('name','asc')->with('parent');
		
    	$posted = $request->all();
		if(!empty($posted)){
			if(!empty($posted['search'])){
    			$specifications->where('name','like','%'.$posted['search'].'%');
    		}
		}
    	
    	$specifications = $specifications->paginate(75);
		
		return view('specifications/index',['specifications'=>$specifications,'posted'=>$posted]);
    }
	
	public function create()
    {
    	/*
    	$catTree = Specification::whereNull('parent_id')->with(['specifications'=>function($query){
    		$query->where('enabled','Yes');
    	}])->get()->toArray();
		
		$catTree = $this->genSpecList($catTree);
		*/
		$catTree = Specification::whereNull('parent_id')->where('enabled','Yes')->get()->toArray();
		
		return view('specifications/create',['parents'=>$catTree]);
    }
	
	public function store(Request $request){
    	
		$request->validate([
            'name' => 'required',
        ],[
        	'name.required'=>"Specification Name is required",
        ]);
		
        $specification = new Specification;
		
		$specification->name = $request->name;
		$specification->parent_id = empty($request->parent_id)?null:$request->parent_id;
		$specification->enabled = $request->enabled;
		//$specification->use_in_filters = $request->use_in_filters;
		
		
		
		if($specification->save()){
			return redirect('/specifications')->with('success','New Specification created successfully!');
		} else {
			return redirect('/specifications')->with('warning','New Specification has not been created!');
		}
    }
	
	
	public function show($id)
    {
        return view('specifications/show');
    }
	
	public function edit($id){
    	$specification = Specification::findOrFail($id);
		
		/*
		$catTree = Specification::whereNull('parent_id')->with(['specifications'=>function($query){
    		$query->where('enabled','Yes');
    	}])->get()->toArray();
		
		$catTree = $this->genSpecList($catTree);
		*/
		
		$catTree = Specification::whereNull('parent_id')->where('id','!=',$id)->where('enabled','Yes')->get()->toArray();
        return view('specifications/edit',['specification'=>$specification,'parents'=>$catTree]);
    }
	
	public function update(Request $request,$id){
    	
		$request->validate([
            'name' => 'required',
        ],[
        	'name.required'=>"Specification Name is required",
        ]);
		
        $specification = Specification::find($id);
			
		$specification->name = $request->name;
		$specification->parent_id = empty($request->parent_id)?null:$request->parent_id;
		$specification->enabled = $request->enabled;
		//$specification->use_in_filters = $request->use_in_filters;
		
		if($specification->save()){
			return redirect('/specifications')->with('success','Specification updated successfully!');
		} else {
			return redirect('/specifications')->with('warning','Specification has not been updated!');
		}
    }
	
	public function destroy($id){
		
		
		$catTree = Specification::where('id',$id)->with('specifications')->get()->toArray();
		$catTree = $this->genSpecChildrenIdsList($catTree);
		
		if(Specification::destroy($catTree)){
			
			return redirect('/specifications')->with('success','Specification deleted successfully!');
		} else {
			return redirect('/specifications')->with('warning','Specification has not been deleted!');
		}
		
		return redirect('/specifications');
		
	}
	
	
	
	public function genSpecList($tree,$saperator="",$cTree=[]){
		
		foreach($tree as $k=>$row){
			$cTree[]=array('id'=>$row['id'],'name'=>$saperator.$row['name']);
			$cTree=$this->genSpecList($row['specifications'],$saperator.$row['name']." &mdash;> ",$cTree);
        }
		
		
		return $cTree;
	}
	
	public function genSpecChildrenIdsList($tree,$cTree=[]){
		
		foreach($tree as $k=>$row){
			$cTree[]=$row['id'];
			$cTree=$this->genSpecChildrenIdsList($row['specifications'],$cTree);
        }
		
		
		return $cTree;
	}
	
}
