<?php

namespace App\Http\Controllers;
use App\Tax;
use Illuminate\Http\Request;
use Validator;
use DB;


class TaxesController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        $taxes = Tax::where('service_charges','No')->get();
        return view('taxes/index',['taxes'=>$taxes]);
    }
	
	public function store(Request $request){
    	
		$request->validate([
            'name' => 'required',
            'rate' => 'required|numeric|max:100'
        ],[
        	'name.required'=>"Tax Name is required",
        	'rate.required'=>"Tax Rate is required",
        	'rate.max'=>"Maximum 100% can be set."
        	
        ]);
		
        $tax = new Tax;
		
		$tax->name = $request->name;
		$tax->rate = $request->rate;
		$tax->enabled = $request->enabled;
		
		
		
		if($tax->save()){
			return redirect('/taxes')->with('success','New Tax created successfully!');
		} else {
			return redirect('/taxes')->with('error','New Tax has not been created!');
		}
    }
	
	
	
	public function edit($id)
    {
    	$tax = Tax::find($id);
		
        $taxes = Tax::get();
        return view('taxes/index',['taxes'=>$taxes,'etax'=>$tax]);
    }
	
	public function update(Request $request,$id){
    	
		$request->validate([
            'name' => 'required',
            'rate' => 'required|numeric|max:100'
        ],[
        	'name.required'=>"Tax Name is required",
        	'rate.required'=>"Tax Rate is required",
        	'rate.max'=>"Maximum 100% can be set."
        	
        ]);
		
        $tax = Tax::find($id);
		
		$tax->name = $request->name;
		$tax->rate = $request->rate;
		$tax->enabled = $request->enabled;
		
		
		
		if($tax->save()){
			return redirect('/taxes')->with('success','Tax updated successfully!');
		} else {
			return redirect('/taxes')->with('error','Tax has not been updated!');
		}
    }
	
	
	public function destroy($id){
		
		if(Tax::destroy($id)){
			return redirect()->back()->with('success','Tax has been Deleted!');
		} else {
			return redirect()->back()->with('error','Tax has not been Deleted!');
		}
	}
	
	/**
	 * Service Charges related actions
	 */
	public function serviceChargesIndex()
	{
	    $taxes = Tax::where('service_charges','Yes')->get();
	    return view('taxes/service-charges',['taxes'=>$taxes]);
	}
	
	public function updateServiceCharges(Request $request)
	{
	    
	    $request->validate([
	        'rate' => 'required|numeric'
	    ],[
	        'rate.required'=>"Price is required"
	        
	    ]);
	    
	    $tax = Tax::find($request->id);
	    
	    $tax->rate = $request->rate;
	    
	    if($tax->save()){
	        return redirect('/service-charges')->with('success','Service charges updated successfully!');
	    } else {
	        return redirect('/service-charges')->with('error','Service charges has not been updated!');
	    }
	}
	
}
