<?php
namespace App\Http\Controllers;

use App\Address;
use App\CountryCode;
use App\Order;
use App\OrderProduct;
use App\Permission;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Route;
use Twilio\Rest\Client;
use Illuminate\Support\Facades\Mail;
use App\Mail\SupportMail;

class UsersController extends Controller
{

    public $User;

    public function __construct()
    {
        $this->middleware('auth');

        $this->User = new User();
    }

    public function adminUsersList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['users'] = User::where($request->ftype, 'like', '%' . $request->stext . '%')->whereNotIn('role', array(
                'Master Admin',
                'Customer',
                'Seller'
            ))->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['users'] = User::whereNotIn('role', array(
                'Master Admin',
                'Customer',
                'Seller'
            ))->paginate(400);
            $data['ftype'] = "";
            $data['stext'] = "";
        }

        return view('users/adminUsersList', $data);
    }

    public function addAdminUser()
    {
        return view('users/addAdminUser');
    }

    public function saveNewAdminUser(Request $request)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'email' => 'unique:users',
            'phone' => 'unique:users'
        ], [
            'email.unique' => 'User already exists with this email',
            'phone.unique' => 'User with this phone number already exists.'
        ]);

        $user = new User();
        $user->name = $posted['name'];
        $user->role = $posted['role'];
        if (! empty($posted['phone'])) {
            $user->country_code = "+" . $posted['country_code'];
            $user->phone = $posted['phone'];
        }
        $user->email = $posted['email'];
        $user->password = Hash::make($posted['password']);
        $user->enabled = $posted['enabled'];

        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $user->photo = $imageUrl;
            }
        }

        if ($user->save()) {
            return redirect()->route('adminUserProfle', $user->id)->with('success', 'User created successfully.');
        } else {
            return back()->with('error', 'User has not been created.');
        }
    }

    public function editAdminUser($id)
    {
        $user = User::where([
            'id' => $id
        ])->whereNotIn('role', [
            'Master Admin',
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('adminUsersList')->with('warning', 'Invalid User to edit');
        }
        return view('users/editAdminUser', [
            'user' => $user
        ]);
    }

    public function saveEditAdminUser(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();

        $request->validate([
            'email' => 'unique:users,id,' . $id,
            'phone' => 'unique:users,id,' . $id
        ], [
            'email.unique' => 'User already exists with this email',
            'phone.unique' => 'User with this Phone already exists.'
        ]);

        $user = User::where([
            'id' => $id
        ])->whereNotIn('role', [
            'Master Admin',
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('adminUsersList')->with('warning', 'Invalid User to edit');
        }
        $user->name = $posted['name'];
        $user->role = $posted['role'];
        $user->country_code = "+" . $posted['country_code'];
        $user->phone = $posted['phone'];
        $user->email = $posted['email'];
        if (! empty($user->password)) {
            $user->password = Hash::make($posted['password']);
        }
        $user->enabled = $posted['enabled'];

        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $user->photo = $imageUrl;
            }
        }

        if ($user->save()) {
            return redirect()->route('adminUserProfle', $user->id)->with('success', 'User updated successfully.');
        } else {
            return back()->with('error', 'User has not been modified.');
        }
    }

    public function adminUserProfle($id = null)
    {
        $user = User::where([
            'id' => $id
        ])->whereNotIn('role', [
            'Master Admin',
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('adminUsersList')->with('warning', 'Invalid User Details');
        }

        return view('users/adminUserProfle', [
            'user' => $user
        ]);
    }

    public function appUsersList(Request $request)
    {
        if (count($request->all()) > 1) {
            $data['users'] = User::where($request->ftype, 'like', '%' . $request->stext . '%')->whereIn('role', array(
                'Customer'
            ))->paginate(400);
            $data['ftype'] = $request->ftype;
            $data['stext'] = $request->stext;
        } else {
            $data['users'] = User::whereIn('role', array(
                'Customer'
            ))->paginate(400);
            $data['ftype'] = "";
            $data['stext'] = "";
        }

        return view('users/appUsersList', $data);
    }

    public function appUserProfle($id = null)
    {
        $user = User::where([
            'id' => $id
        ])->whereIn('role', [
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('appUsersList')->with('warning', 'Invalid User Details');
        }

        $orders = Order::where('user_id', $id)->whereHas('order_products')->with('order_products.order_status')
            ->orderBy('created_at', 'desc')
            ->paginate(10);

        $stats = [
            'total' => OrderProduct::whereHas('order', function ($q) use ($id) {
                $q->where('user_id', $id);
            })->count(),
            'delivered' => OrderProduct::whereHas('order', function ($q) use ($id) {
                $q->where('user_id', $id);
            })->where('status', 'Delivered')->count(),
            'pending' => OrderProduct::whereHas('order', function ($q) use ($id) {
                $q->where('user_id', $id);
            })->where('status', 'Pending')->count(),
            'cancelled' => OrderProduct::whereHas('order', function ($q) use ($id) {
                $q->where('user_id', $id);
            })->where('status', 'Rejected')->count()
        ];

        return view('users/appUserProfle', [
            'user' => $user,
            'orders' => $orders,
            'stats' => $stats
        ]);
    }

    public function editAppUser($id)
    {
        $user = User::where([
            'id' => $id
        ])->whereIn('role', [
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('appUsersList')->with('warning', 'Invalid User to edit');
        }

        $country_codes = CountryCode::get();

        return view('users/editAppUser', [
            'user' => $user,
            'country_codes' => $country_codes
        ]);
    }

    public function saveEditAppUser(Request $request, $id)
    {
        $posted = collect($request->all())->toArray();

        // dd($posted);

        $request->validate([
            'name' => 'required',
            'gender' => 'required',
            'email' => 'email:strict,rfc,dns|unique:users,id,' . $id,
            'phone' => 'unique:users,id,' . $id,
            'country_code' => 'required'
        ], [
            'name.required' => 'Display Name is required',
            'gender.required' => 'Gender is required',
            'email.unique' => 'User already exists with this email',
            'phone.unique' => 'User with this Phone already exists.',
            'email.email' => "Invalid email provided."
        ]);

        $user = User::where([
            'id' => $id
        ])->whereIn('role', [
            'Customer'
        ])->first();
        if (! $user) {
            return redirect()->route('appUsersList')->with('warning', 'Invalid User to edit');
        }
        $user->name = $posted['name'];
        $user->country_code = $posted['country_code'];
        $user->phone = $posted['phone'];
        $user->email = $posted['email'];
        $user->gender = $posted['gender'];
        $user->enabled = $posted['enabled'];

        // dd($_FILES);
        if ($request->hasFile('photo')) {
            $image = $request->file('photo');
            $imageFileName = str_replace(" ", "", str_replace(".", "", microtime())) . '.' . $image->getClientOriginalExtension();
            $imageUrl = "https://ojaprod.s3.us-east-2.amazonaws.com/users/" . $imageFileName;
            $s3 = \Storage::disk('s3');
            $filePath = '/users/' . $imageFileName;
            $imageResult = $s3->put($filePath, file_get_contents($image), 'public');
            if ($imageResult == 1) {
                $user->photo = $imageUrl;
            }
        }

        if ($user->save()) {
            return redirect()->route('appUserProfle', $user->id)->with('success', 'User updated successfully.');
        } else {
            return back()->with('error', 'User has not been modified.');
        }
    }

    public function deleteAppUser(Request $request)
    {
        if (User::find($request->id)->forceDelete()) {

            Address::where('user_id', $request->id)->delete();

            return redirect()->back()->with('success', 'User has been Deleted!');
        } else {
            return redirect()->back()->with('error', 'User has not been Deleted!');
        }
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('users/index');
    }

    /*
     * public function show($id)
     * {
     *
     *
     * return view('uesrs/show');
     * }
     *
     */
    
    public function rolePermissions(Request $request, $role = null)
    {
       
            if ($request->isMethod('post')) {
                $posted = $request->all();
                $flag = 0;
                foreach ($posted['route'] as $routeName => $permission) {

                    if (! empty($permission['write'])) {
                        $perm = 2;
                    } elseif (! empty($permission['read'])) {
                        $perm = 1;
                    } else {
                        $perm = 0;
                    }
                    $upid = empty($permission['id']) ? "" : $permission['id'];

                    $userPermissions = [
                        'role' => $role,
                        'route_name' => $routeName,
                        'permission' => $perm
                    ];

                    if (! empty($userPermissions)) {
                        DB::beginTransaction();

                        try {
                            Permission::updateOrCreate([
                                'id' => $upid
                            ], $userPermissions);
                            DB::commit();
                            $flag ++;
                        } catch (\Exception $e) {
                            DB::rollback();
                        }
                    }
                }

                if ($flag == count($posted['route'])) {
                    return back()->with('success', "Successfully Updated.");
                } else {
                    return back()->with('error', "Not Successful. Try again.");
                }
            }
            

            $upermissions = [];
            $routeNames = [];
            if ($role != null) { // getting selected user's permissions
                $routes = collect(Route::getRoutes())->map(function ($route) {
                    return [
                        'name' => $route->getName(),
                        'action' => $route->getActionName()
                    ];
                });

                $toOmmit = [
                    'passport',
                    'logout',
                    'login',
                    'register',
                    'rolePermissions'
                ];
                foreach ($routes as $k => $route) {
                    if (! empty($route['name'])) {
                        $rname = explode('.', $route['name']);
                        if (! in_array($rname[0], $toOmmit)) {
                            $routeNames[$k] = $route['name'];
                        }
                    }
                }

                $uperms = Permission::where('role', $role)->get();
                foreach ($uperms as $uperm) {
                    $upermissions[$uperm->route_name] = $uperm;
                }
                // dd($uperms);
            }

            $data = [
                'routes' => $routeNames,
                'permissions' => $upermissions,
                'role' => $role
            ];

            return view('users/role_permissions', $data);
        
    } 
    
    public function updateNotificationToken(Request $request)
    {
        if(!empty($request->player_id)){
            $user = Auth::user();
            
            if(!empty($user)){
                $user->device_token = $request->player_id;
                $user->save();

                $title = "New Token Saved ";
                $data = ['notification_type'=>'New Token Saved.'];
                User::sendNotification($user->device_token,$title,$data);
                echo "user saved";
            }
            echo "no user saved"; 
        }
    }
    
    //turn notification on or off
    public function changeNotificationStatus()
    {
        $user = Auth::user();
        
        if(!empty($user))
        {
            if($user->notifications == 'Yes'){
                $user->notifications = 'No';
                $status = "Disabled";
            }else{
                $user->notifications = 'Yes';
                $status = "Enabled";
            }
            
            if($user->save()){
                return response()->json(['status' => 1, 'message' => 'Notification '. $status . ' successfully.']);
            }else{
                return response()->json(['status' => 0, 'message' => 'Notification does not '. $status .' successfully.']);
            }
        }
    }
    
    
}
