<?php

namespace App\Imports;

use App\Product;
use App\Category;
use App\Merchant;
use Illuminate\Validation\Rule;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\ToModel;
use Maatwebsite\Excel\Validators\Failure;
use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\SkipsOnFailure;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Concerns\SkipsFailures;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\WithStartRow;
use Maatwebsite\Excel\Concerns\WithBatchInserts;



class ProductImport implements ToModel, WithStartRow, WithValidation, WithBatchInserts	{
    use Importable;
	
	public function startRow(): int {
        return 2;
    }
	
	public function batchSize(): int {
        return 100;
    }
	
	/*
	public function collection(Collection $rows) {
		
		 Validator::make($rows->toArray(), [
             '0' => 'required|numeric',
             '2' => 'required|email',
             '3' => 'required',
             '5' => 'required',
             '7' => 'numeric',
             '8' => 'required|numeric',
             '9' => 'numeric',
             '11' => 'required|numeric',
             '12' => 'required',
             '14' => 'required|numeric',
             '15' => 'required'
         ])->validate();
		
		
        foreach ($rows as $key=>$row) {
        	
			//getting merchant id
			$mer = Merchant::where('email',$row[2])->get()->first();
			if($mer){
				$row[2] = $mer->id;
			} else {
				continue;
			}
			
            Product::create([
                'category_id'=> $row[0],
	            'merchant_id'=>$row[2],
	            'name'    => $row[3],
	            'short_description'=>$row[4],
	            'sku'=>$row[5],
	            'manage_stock'=>empty($row[6])?'No':$row[6],
	            'stock_quantity'=>$row[7],
	            'min_order_quantity'=>$row[8],
	            'weight'=>$row[9],
	            'unit'=>$row[10],
	            'regular_price'=>$row[11],
	            'sale_price'=>$row[11],
	            'image'=>$row[12],
	            'tags'=>$row[13],
	            'sort_order'=>$row[14],
	            'enabled'=>$row[15]
            ]);
        }
    }
	*/
	
    public function model(array $row)
    {
    	//getting merchant id
		$mer = Merchant::where('email',$row[2])->get()->first();
		
		if($mer){
			$row[2] = $mer->id;
		} else {
			$row[2]="";
		}
		
        return new Product([
            'category_id'=> $row[0],
            'merchant_id'=>$row[2],
            'name'    => $row[3],
            'short_description'=>$row[4],
            'sku'=>$row[5],
            'manage_stock'=>$row[6],
            'stock_quantity'=>$row[7],
            'min_order_quantity'=>$row[8],
            'weight'=>$row[9],
            'unit'=>$row[10],
            'regular_price'=>$row[11],
            'sale_price'=>$row[11],
            'image'=>$row[12],
            'tags'=>$row[13],
            'sort_order'=>$row[14],
            'enabled'=>$row[15]
        ]);
    }

    public function rules(): array {
        return [
            
			'0' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Category ID is required');
			 	}
				if(!is_numeric($value)){
					$onFailure('Invalid Category ID Provided');
				}
			},
			'3' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Product Name is required');
			 	}
			},
			
            '2'=>function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Merchant email is required');
			 	}
				if(!filter_var($value, FILTER_VALIDATE_EMAIL)){
					$onFailure('Invalid Merchant Email.');
				}
				
				//checking if merchant exists or not
				$mer = Merchant::where('email',$value)->get()->first();
				if(!$mer){
					$onFailure('Merchant with Email '.$value.' does not exists.');
				}

			},
			'3' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Product Name is Required');
			 	}
			},
			'5' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('SKU is required');
			 	}
			},
			'6' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Manage Stock Yes/No is required');
			 	}
			},
			
			'10' => function($attribute, $value, $onFailure){
				$unts = ['KG','Grams',''];
			 	if(!in_array($value,$unts)){
					$onFailure('Only KG or Grams are valid for Weight Unit.');
				}
			},
			'11' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Product Price is required');
			 	}
				if(!is_numeric($value)){
					$onFailure('Only Numbers are valid for Price');
				}
			},
			
			'12' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Product Image is Required');
			 	}
				if(!filter_var($value, FILTER_VALIDATE_URL)){
					$onFailure('Only URL is valid for Image');
				}
			},
			'14' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Sort Order is required');
			 	}
				if(!is_numeric($value)){
					$onFailure('Only Numbers are valid for Sort Order');
				}
			},
			
			'15' => function($attribute, $value, $onFailure){
			 	if(empty($value)){
			 		$onFailure('Enabled Yes/No is required');
			 	}
			}
        ];
    }
	 
	 

	
}
