<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    const PAYMENT_TYPE_CREDIT_CARD = "CC";
    const PAYMENT_TYPE_DEBIT_CARD = "DC";
    const PAYMENT_TYPE_CASH_ON_DELIVERY = "COD";
    
    const STATE_PENDING = "Pending";
    const STATE_CANCELED = "Canceled";
    const STATE_CONFIRMED = "Confirmed";
    const STATE_PROCESSING = "Processing";
    const STATE_READY_TO_PICKUP = "Ready to Pickup";
    const STATE_OUT_FOR_DELIVERY = "Out for Delivery";
    const STATE_REJECTED = "Rejected";
    const STATE_DELIVERED = "Delivered";
    
    protected $guarded = [];
	protected $appends = ['delivery_address'];
	protected $table = "orders";
	protected $hidden = ['address'];
	
	//getting order products
	public function order_products(){
		return $this->hasMany('App\OrderProduct','order_id');
	}
	
	//getting statuses
	public function order_status(){
	    return $this->hasMany('App\OrderStatus','order_id');
	}
	
	public function user(){
		return $this->belongsTo('App\User','user_id');
	}
	
	//accessor and mutators
	public function getDeliveryAddressAttribute(){
   		return  json_decode($this->address);
	}
	
	public static function getOrderStatuses()
	{
	    $data = [
	        self::STATE_PENDING => 'Pending',
	        self::STATE_CONFIRMED => 'Confirmed',
	        self::STATE_PROCESSING => 'Processing',
	        self::STATE_READY_TO_PICKUP => 'Ready to Pickup',
	        self::STATE_OUT_FOR_DELIVERY => 'Out for Delivery',
	        self::STATE_DELIVERED => 'Delivered',
	        self::STATE_REJECTED => 'Rejected'
	    ];
	    
	    return $data;
	}
	
	
}
